@@ -36,22 +36,11 @@ static int ref_familySerif = 0;
 static int ref_familySansSerif = 0;
 static int ref_familyMonospace = 0;
 
-/*
-	How font cache works
+extern cairo_font_face_t *
+_cairo_toy_font_face_create (const char *family, 
+			     cairo_font_slant_t   slant, 
+			     cairo_font_weight_t  weight);
 
-	- Create any font and store it in the cache
-	- When GdipDeleteFont is called just decrease the reference count and do not delete the font
-	- All the cached fonts are released when GDI+ is shutdown
-	- When the cache is full we release the first font cached with 0 references
-*/
-
-#define MAX_CACHED_FONTS 128
-static GpCachedFont cached_fonts [MAX_CACHED_FONTS];
-static int cached_fonts_index = 0;
-static pthread_mutex_t	fontcache_mutex = PTHREAD_MUTEX_INITIALIZER;
-
-#define	LOCK_FONTCACHE		pthread_mutex_lock(&fontcache_mutex)
-#define	UNLOCK_FONTCACHE	pthread_mutex_unlock(&fontcache_mutex)
 
 /* Family and collections font functions */
 
@@ -72,7 +61,7 @@ GpStatus
 GdipNewInstalledFontCollection (GpFontCollection **font_collection)
 {	
 	
-	FcObjectSet *os = FcObjectSetBuild (FC_FAMILY, FC_FOUNDRY, 0);
+	FcObjectSet *os = FcObjectSetBuild (FC_FAMILY, FC_FOUNDRY, NULL);
 	FcPattern *pat = FcPatternCreate ();
 	FcValue val;
 	FcFontSet *col;
@@ -138,12 +127,11 @@ GdipPrivateAddFontFile (GpFontCollection *font_collection,  GDIPCONST WCHAR *fil
 	if (!font_collection || !filename)
 		return InvalidParameter;
     
-	file = (unsigned char*) g_utf16_to_utf8 ((const gunichar2 *)filename, -1,
-						 NULL, NULL, NULL);
-	
+	file = (unsigned char*) ucs2_to_utf8 ((const gunichar2 *)filename, -1);
+
 	FcConfigAppFontAddFile (font_collection->config, file);
     
-	g_free (file);
+	GdipFree (file);
 	return Ok;
 }
 
@@ -185,7 +173,7 @@ GdipDeleteFontFamily (GpFontFamily *fontFamily)
 void
 gdip_createPrivateFontSet (GpFontCollection *font_collection)
 {
-	FcObjectSet *os = FcObjectSetBuild (FC_FAMILY, FC_FOUNDRY, 0);
+	FcObjectSet *os = FcObjectSetBuild (FC_FAMILY, FC_FOUNDRY, NULL);
 	FcPattern *pat = FcPatternCreate ();
 	FcFontSet *col =  FcFontList (font_collection->config, pat, os);
     
@@ -221,13 +209,12 @@ GdipGetFontCollectionFamilyList (GpFontCollection *font_collection, int num_soug
 	GpFontFamily **gpfam = gpfamilies;
 	FcPattern **pattern =  font_collection->fontset->fonts;
 	int i;
-	
+
 	if (!font_collection || !gpfamilies || !num_found)
 		return InvalidParameter;
 
 	if (font_collection->config)
 		gdip_createPrivateFontSet (font_collection);
-		
 
 	for (i = 0; i < font_collection->fontset->nfont; gpfam++, pattern++, i++) {
 		gdip_createFontFamily (gpfam);
@@ -242,8 +229,6 @@ GdipGetFontCollectionFamilyList (GpFontCollection *font_collection, int num_soug
 GpStatus
 GdipCreateFontFamilyFromName (GDIPCONST WCHAR *name, GpFontCollection *font_collection, GpFontFamily **fontFamily)
 {
-	glong items_read = 0;
-	glong items_written = 0;
 	unsigned char *string;
 	FcPattern **gpfam;
 	FcChar8 *str;
@@ -252,14 +237,11 @@ GdipCreateFontFamilyFromName (GDIPCONST WCHAR *name, GpFontCollection *font_coll
 	if (!name || !fontFamily)
 		return InvalidParameter;
 
-	string = (unsigned char*)g_utf16_to_utf8 ((const gunichar2 *)name, -1,
-						  &items_read, &items_written, NULL);
+	string = (unsigned char*)ucs2_to_utf8 ((const gunichar2 *)name, -1);
 
 	if (!font_collection) {
-		FcChar8 *str;
 		FcPattern *pat = FcPatternCreate ();
 		FcResult rlt;
-		FcResult r;
 		
 		/* find the family we want */
 		FcValue val;
@@ -274,56 +256,46 @@ GdipCreateFontFamilyFromName (GDIPCONST WCHAR *name, GpFontCollection *font_coll
 		gdip_createFontFamily (fontFamily);
 		(*fontFamily)->pattern =  FcFontMatch (0, pat, &rlt);
 		(*fontFamily)->allocated = TRUE;
-
-		r = FcPatternGetString ((*fontFamily)->pattern, FC_FAMILY, 0, &str);
-		g_free (string);
-		FcPatternDestroy (pat);
+		GdipFree (string);
+		if ((*fontFamily)->pattern == NULL) {
+			(*fontFamily)->pattern = pat;
+		} else {
+			FcPatternDestroy (pat);			
+		}
+		
 		return Ok;
 	}
 
 	gpfam = font_collection->fontset->fonts;
     
 	for (i=0; i < font_collection->fontset->nfont; gpfam++, i++){
-		FcResult r = FcPatternGetString (*gpfam, FC_FAMILY, 0, &str);
+		FcPatternGetString (*gpfam, FC_FAMILY, 0, &str);
 
 		if (strcmp ((char *)string, (const char *)str)==0) {
 			gdip_createFontFamily (fontFamily);
 			(*fontFamily)->pattern = *gpfam;
 			(*fontFamily)->allocated = FALSE;
-			g_free (string);
+			GdipFree (string);
 			return Ok;
 		}
 	}
 
-	g_free (string);
+	GdipFree (string);
 	return FontFamilyNotFound;
 }
 
-
 GpStatus
 GdipGetFamilyName (GDIPCONST GpFontFamily *family, WCHAR name[LF_FACESIZE], int language)
 {                
 	FcChar8 *fc_str;
-	glong items_read = 0;
-	glong items_written = 0;
 	FcResult r;
-	gunichar2 *string;
 	
 	if (!family)
 		return InvalidParameter;
 
 	r = FcPatternGetString (family->pattern, FC_FAMILY, 0, &fc_str);
 
-	string =  g_utf8_to_utf16 ((const gchar *)fc_str, -1, &items_read, &items_written,NULL);
-
-	if (items_written>= (LF_FACESIZE-1))
-		items_written= (LF_FACESIZE-1);
-
-	memcpy (name, string, items_written * sizeof (WCHAR));
-	name [1+items_written*sizeof (WCHAR)]=0;
-
-	g_free (string);
-
+	utf8_to_ucs2((const gchar *)fc_str, (gunichar2 *)name, LF_FACESIZE);
 	return Ok;
 }
 
@@ -367,6 +339,39 @@ GdipGetGenericFontFamilyMonospace (GpFontFamily **nativeFamily)
 	return Ok;
 }
 
+FT_Face
+gdip_cairo_ft_font_lock_face (cairo_font_face_t *cairofnt)
+{
+	cairo_scaled_font_t *scaled_ft;
+	cairo_matrix_t matrix1, matrix2;
+        cairo_font_options_t *options;
+	
+	options = cairo_font_options_create ();
+		
+	scaled_ft = cairo_scaled_font_create (cairofnt,
+					      &matrix1,
+					      &matrix2,
+					      options);
+	return cairo_ft_scaled_font_lock_face (scaled_ft);
+}
+
+void
+gdip_cairo_ft_font_unlock_face (cairo_font_face_t *cairofnt)
+{
+	cairo_scaled_font_t *scaled_ft;
+	cairo_matrix_t matrix1, matrix2;
+	cairo_font_options_t *options = NULL;
+
+	options = cairo_font_options_create ();	
+	
+	scaled_ft = cairo_scaled_font_create (cairofnt,
+					      &matrix1,
+					      &matrix2,
+					      options);
+	
+	cairo_ft_scaled_font_unlock_face (scaled_ft);
+}
+
 GpStatus
 GdipGetEmHeight (GDIPCONST GpFontFamily *family, GpFontStyle style, short *EmHeight)
 {
@@ -528,117 +533,18 @@ GdipIsStyleAvailable (GDIPCONST GpFontFamily *family, int style, BOOL *IsStyleAv
 	return Ok;    
 }
 
-
 /* Font functions */
 
-int
-gdip_font_create (const unsigned char *family, int fcslant, int fcweight, GpFont *result)
-{
-	cairo_font_t *font = NULL;
-	FcPattern * pat = NULL;
-	FT_Library ft_library;
-	FT_Error error;
-
-	pat = FcPatternCreate ();
-	if (pat == NULL || result == NULL) {
-		return 0;
-	}
-
-	FcPatternAddString (pat, FC_FAMILY, family);
-	FcPatternAddInteger (pat, FC_SLANT, fcslant);
-	FcPatternAddInteger (pat, FC_WEIGHT, fcweight);
-
-	error = FT_Init_FreeType (&ft_library);
-	if (error) {
-		FcPatternDestroy (pat);
-		return 0;
-	}
-
-	font = cairo_ft_font_create (ft_library, pat);
-	if (font == NULL) {
-		FT_Done_FreeType(ft_library);
-		FcPatternDestroy (pat);
-		return 0;
-	}
-
-	result->cairofnt = font;
-	result->ft_library = ft_library;
-
-	FT_Set_Char_Size (gdip_cairo_ft_font_lock_face(font),
-			  DOUBLE_TO_26_6 (1.0),
-			  DOUBLE_TO_26_6 (1.0),
-			  0, 0);
-	gdip_cairo_ft_font_unlock_face(font);
-
-	FcPatternDestroy (pat);
-	return 1;
-}
-
-void
-gdip_font_drawunderline (GpGraphics *graphics, GpBrush *brush, float x, float y, float width)
-{
-        float pos, size;
-        cairo_font_extents_t extents;
-
-        cairo_current_font_extents (graphics->ct, &extents);
-        pos = 0.5 + ((extents.ascent + extents.descent) *0.1);
-        size = 0.5 + ((extents.ascent + extents.descent) *0.05);
-
-        GdipFillRectangle (graphics, brush, x, y +pos, width, size);     
-}
-
-void
-gdip_font_drawstrikeout (GpGraphics *graphics, GpBrush *brush, float x, float y, float width)
-{
-        float pos, size;
-        cairo_font_extents_t extents;
-
-        cairo_current_font_extents (graphics->ct, &extents);
-        pos = 0.5 + ((extents.ascent + extents.descent) *0.5);
-        size = 0.5 + ((extents.ascent + extents.descent) *0.05);
-
-        GdipFillRectangle (graphics, brush, x, y -pos, width, size);
-}
-
-void
-gdip_release_font (GpFont* font)
-{
-	if (!font)
-		return;
-
-	cairo_font_destroy ((cairo_font_t *)font->cairofnt);
-
-	if (font->ft_library)
-		FT_Done_FreeType(font->ft_library);
-	
-	GdipFree ((void *)font);	
-}
-
-void
-gdip_release_cachedfonts ()
-{
-	int i;
-
-	/* Loop all the cached fonts */
-	
-	LOCK_FONTCACHE;
-	for (i = 0; i < cached_fonts_index; i++) {
-		gdip_release_font (cached_fonts[i].font);
-	}
-	UNLOCK_FONTCACHE;
-}
-
-
 GpStatus
 GdipCreateFont (GDIPCONST GpFontFamily* family, float emSize, GpFontStyle style, Unit unit,  GpFont **font)
 {
 	FcChar8* str;
 	FcResult r;
 	GpFont *result;
-	int slant = 0;
-	int weight = FC_WEIGHT_LIGHT;
-	int i;
+	cairo_font_slant_t slant;
+	cairo_font_weight_t weight;
 	float sizeInPixels;
+	cairo_font_face_t* cairofnt;
 	
 	if (!family || !font)
 		return InvalidParameter;
@@ -646,76 +552,28 @@ GdipCreateFont (GDIPCONST GpFontFamily* family, float emSize, GpFontStyle style,
 	r = FcPatternGetString (family->pattern, FC_FAMILY, 0, &str);
 	
 	gdip_unitConversion (unit, UnitPixel, emSize, &sizeInPixels);
-
-	
-	/* Is it already in the cache */
-	LOCK_FONTCACHE;
-	for (i = 0; i < cached_fonts_index; i++) {
-		if (sizeInPixels != cached_fonts[i].sizeInPixels)
-			continue;
-
-		if (style != cached_fonts[i].style)
-			continue;
 		
-		if (strcmp ((char *)str, (const char *)cached_fonts[i].szFamily) != 0)
-			continue;
-
-		if (cached_fonts[i].font == NULL)
-			continue;
-
-		/* Found in cache */
-		*font = cached_fonts[i].font;
-		cached_fonts[i].refcount++;
-		UNLOCK_FONTCACHE;
-		return Ok;
-	}
-	UNLOCK_FONTCACHE;
-
 	result = (GpFont *) GdipAlloc (sizeof (GpFont));
 	result->sizeInPixels = sizeInPixels;
 
         if ((style & FontStyleBold) == FontStyleBold)
-                weight = FC_WEIGHT_BOLD;
+                weight = CAIRO_FONT_WEIGHT_BOLD;
+	else
+		weight = CAIRO_FONT_WEIGHT_NORMAL;
 
         if ((style & FontStyleItalic) == FontStyleItalic)        
-                slant = FC_SLANT_ITALIC;
-        
-	if (!gdip_font_create (str, slant, weight, result)) {
-		return InvalidParameter;	/* FIXME -  wrong return code */
-	}
+                slant = CAIRO_FONT_SLANT_ITALIC;
+	else
+		slant = CAIRO_FONT_SLANT_NORMAL;
+
+	cairofnt = _cairo_toy_font_face_create ((const char*) str, slant, weight);
+
+	if (cairofnt == NULL)
+		return GenericError;	
+
+	result->cairofnt = cairofnt;
         result->style = style;
-	cairo_font_reference ((cairo_font_t *)result->cairofnt);
-	*font=result;
-
-	if (strlen ((const char *)str) > 127) /* Cannot cache this font */
-		return Ok;
-	
-	/* Cache entry */
-	LOCK_FONTCACHE;
-	if (cached_fonts_index < MAX_CACHED_FONTS) {
-		strcpy (cached_fonts[cached_fonts_index].szFamily, (const char *)str);
-		cached_fonts[cached_fonts_index].sizeInPixels = sizeInPixels;
-		cached_fonts[cached_fonts_index].style = style;
-		cached_fonts[cached_fonts_index].font = result;
-		cached_fonts[cached_fonts_index].refcount = 1;
-		cached_fonts_index++;
-	} else {
-
-		/* Look for the first non-referenced font */
-		for (i = 0; i < MAX_CACHED_FONTS; i++) {
-			if (cached_fonts[i].refcount > 0)
-				continue;
-
-			gdip_release_font (cached_fonts[i].font); /* release previous cached font */
-			strcpy (cached_fonts[i].szFamily, (const char *)str);
-			cached_fonts[i].sizeInPixels = sizeInPixels;
-			cached_fonts[i].style = style;
-			cached_fonts[i].font = result;
-			cached_fonts[i].refcount = 1;
-		}	
-	}
-	UNLOCK_FONTCACHE;
-        
+	*font = result;	        		
 	return Ok;
 }
 
@@ -733,25 +591,11 @@ GdipGetHfont(GpFont* font, void **Hfont)
 GpStatus
 GdipDeleteFont (GpFont* font)
 {
-	int i;
-
 	if (!font)
 		return InvalidParameter;
 
-	/* Is is in the cache */
-	LOCK_FONTCACHE;
-	for (i = 0; i < cached_fonts_index; i++) {
-		if (font != cached_fonts[i].font)
-			continue;		
-
-		/* Found in cache */
-		cached_fonts[i].refcount--;
-		UNLOCK_FONTCACHE;
-		return Ok;
-	}
-	UNLOCK_FONTCACHE;
-
-	gdip_release_font (font);
+	cairo_font_face_destroy (font->cairofnt);
+	GdipFree ((void *)font);
 	return Ok;	       
 }
 
@@ -811,17 +655,16 @@ GetFontMetrics(GpGraphics *graphics, GpFont *font, int *ascent, int *descent)
 {
 	cairo_font_extents_t	font_extent;
 
-	cairo_set_font (graphics->ct, (cairo_font_t*) font->cairofnt);
-	cairo_scale_font (graphics->ct, font->sizeInPixels);
-	cairo_current_font_extents (graphics->ct, &font_extent);
+	cairo_set_font_face (graphics->ct, (cairo_font_face_t*) font->cairofnt);
+	cairo_set_font_size (graphics->ct, font->sizeInPixels);
+	cairo_font_extents (graphics->ct, &font_extent);
 
 	if (ascent) {
 		*ascent = (int)font_extent.ascent;
 	}
 
-	/* Descent is negative for descent below the baseline */
 	if (descent) {
-		*descent = -1 * (int)font_extent.descent;
+		*descent = (int)font_extent.descent;
 	}
 
 	return TRUE;
