@@ -9,6 +9,7 @@
  *  	Sanjay Gupta (gsanjay@novell.com)
  *	Vladimir Vukicevic (vladimir@pobox.com)
  *	Geoff Norton (gnorton@customerdna.com)
+ *      Jonathan Gilbert (logic@deltaq.org)
  *
  * Copyright (C) Novell, Inc. 2003-2005. http://www.novell.com
  */
@@ -40,35 +41,34 @@
 #endif
 #endif
 
-#define	gdip_cairo_ft_font_lock_face(font)	cairo_ft_font_face(font)
-#define gdip_cairo_ft_font_unlock_face(font)
+#define gdip_cairo_matrix_copy(m1, m2) memcpy (m1, m2, sizeof (cairo_matrix_t))
 
 #ifdef WORDS_BIGENDIAN
-#define set_pixel_bgra(pixel,index,b,g,r,a) { \
-		pixel[index+0] = a; \
-		pixel[index+1] = r; \
-		pixel[index+2] = g; \
-		pixel[index+3] = b; \
-	}
-#define get_pixel_bgra(color, b, g, r, a) { \
+#define set_pixel_bgra(pixel,index,b,g,r,a) do { \
+		((unsigned char *)(pixel))[index+0] = a; \
+		((unsigned char *)(pixel))[index+1] = r; \
+		((unsigned char *)(pixel))[index+2] = g; \
+		((unsigned char *)(pixel))[index+3] = b; \
+	} while (0)
+#define get_pixel_bgra(color, b, g, r, a) do { \
 		a = (color & 0x000000ff); \
 		r = (color & 0x0000ff00) >> 8; \
 		g = (color & 0x00ff0000) >> 16; \
 		b = (color & 0xff000000) >> 24; \
-	}
+	} while (0)
 #else
-#define set_pixel_bgra(pixel,index,b,g,r,a) { \
-		pixel[index+0] = b; \
-		pixel[index+1] = g; \
-		pixel[index+2] = r; \
-		pixel[index+3] = a; \
-	}
-#define get_pixel_bgra(color, b, g, r, a) { \
+#define set_pixel_bgra(pixel,index,b,g,r,a) do { \
+		((unsigned char *)(pixel))[index+0] = b; \
+		((unsigned char *)(pixel))[index+1] = g; \
+		((unsigned char *)(pixel))[index+2] = r; \
+		((unsigned char *)(pixel))[index+3] = a; \
+	} while (0)
+#define get_pixel_bgra(color, b, g, r, a) do { \
 		a = ((color & 0xff000000) >> 24); \
 		r = ((color & 0x00ff0000) >> 16); \
 		g = ((color & 0x0000ff00) >> 8); \
 		b = (color & 0x000000ff); \
-	}
+	} while(0)
 #endif
 
 #ifdef CAIRO_HAS_XLIB_SURFACE
@@ -143,6 +143,9 @@ typedef ImageAbort GetThumbnailImageAbort;
 #define CURVE_MIN_TERMS 1
 #define CURVE_MAX_TERMS 7
 
+#define CAIRO_AA_OFFSET_X	1
+#define CAIRO_AA_OFFSET_Y	0.5
+
 typedef unsigned char byte;
 typedef int bool;
 typedef guint16 WCHAR; /* 16-bits unicode */
@@ -424,7 +427,6 @@ typedef enum {
 typedef struct {
 	unsigned long	Flags;
 	unsigned long	Linefeeds;
-	float		TabWidth;
 	float		PosX;		/* We call it X, even though it might become Y for vertical drawing */
 	float		PosY;		/* We call it Y, even though it might become X for vertical drawing */
 	float		Width;		/* Width of the character; height is defined in font structure */
@@ -681,6 +683,7 @@ typedef cairo_matrix_t GpMatrix;
 typedef struct {
 	int color;
         GpBrush *brush;
+	BOOL own_brush;
 	float width;
         float miter_limit;
         GpLineJoin line_join;
@@ -699,9 +702,25 @@ typedef struct {
         BOOL changed; /* flag to mark if pen is changed and needs setup */
 } GpPen;
 
+ 
+typedef struct {
+	cairo_matrix_t		matrix;
+	GpRegion*		clip;
+	GpCompositingMode    	composite_mode;
+	GpCompositingQuality 	composite_quality;
+	InterpolationMode 	interpolation;
+	GpUnit			page_unit;
+	float			scale;
+	SmoothingMode		draw_mode;
+	TextRenderingHint 	text_mode;
+	PixelOffsetMode 	pixel_mode;
+	int			org_x;
+	int			org_y;
+} GpState;
+
 typedef struct {
 	cairo_t         *ct;
-	cairo_matrix_t	*copy_of_ctm;
+	GpMatrix	*copy_of_ctm;
 	Display		*display;
 	Drawable	drawable;
 	void            *image;
@@ -713,14 +732,20 @@ typedef struct {
 	InterpolationMode interpolation;
 	GpPen*		last_pen;	/* caching pen and brush to avoid unnecessary sets */
 	GpBrush*	last_brush;
-	
+	SmoothingMode	draw_mode;
+	TextRenderingHint text_mode;
+	GpState*	saved_status;
+	int		saved_status_pos;
 	GpCompositingMode    composite_mode;
 	GpCompositingQuality composite_quality;
+	PixelOffsetMode pixel_mode;
+	float		aa_offset_x;
+	float		aa_offset_y;
+	int		render_origin_x;
+	int		render_origin_y;
 } GpGraphics;
 
-typedef struct {
-	cairo_matrix_t		*matrix;
-} GpState;
+
 
 typedef struct {
 	UINT Flags;
@@ -824,11 +849,10 @@ typedef struct {
 
 
 typedef struct {
-        cairo_font_t*       cairofnt;
+        cairo_font_face_t*  cairofnt;
         float               sizeInPixels;
         GpFontStyle         style;
         void                *wineHfont;
-	FT_Library          ft_library;
 } GpFont;
 
  
@@ -900,7 +924,28 @@ typedef struct {
         const BYTE* SigMask;
 } ImageCodecInfo;
 
-                
+/*
+ * Pixel Streams
+ *
+ */
+
+typedef struct
+{
+	Rect region;
+	int x, y;               /* the offset of the next byte that will be loaded, once the buffer is depleted */
+	unsigned short buffer;
+	int p;                  /* index of pixel within 'buffer' that was returned by the last call to gdip_pixel_stream_get_next () */
+	int one_pixel_mask, one_pixel_shift, pixels_per_byte;
+
+	BitmapData *data;
+	unsigned char *scan;
+} StreamingState;
+
+GpStatus gdip_init_pixel_stream (StreamingState *state, BitmapData *data, int x, int y, int w, int h);
+BOOL gdip_pixel_stream_has_next (StreamingState *state);
+unsigned int gdip_pixel_stream_get_next (StreamingState *state);
+void gdip_pixel_stream_set_next (StreamingState *state, unsigned int pixel_value);
+
 /*
  * Functions
  * 
@@ -912,10 +957,8 @@ GpBitmap *gdip_bitmap_new   (void);
 void gdip_bitmap_dispose (GpBitmap *bitmap);
 void gdip_bitmap_clone (GpBitmap *bitmap, GpBitmap **clonedbitmap);
 
-void gdip_graphics_init (GpGraphics *graphics);
-GpGraphics *gdip_graphics_new (void);
-void gdip_graphics_attach_bitmap (GpGraphics *graphics, GpBitmap *image);
-void gdip_graphics_detach_bitmap (GpGraphics *graphics, GpBitmap *image);
+void gdip_graphics_init (GpGraphics *graphics, cairo_surface_t *surface);
+GpGraphics *gdip_graphics_new (cairo_surface_t *surface);
 
 GpStatus gdip_bitmap_clone_data_rect (GdipBitmapData *srcData, Rect *srcRect, GdipBitmapData *destData, Rect *destRect);
 GpStatus gdip_bitmap_change_rect_pixel_format (GdipBitmapData *srcData, Rect *srcRect, GdipBitmapData *destData, Rect *destRect);
@@ -934,6 +977,8 @@ void gdip_process_bitmap_attributes (GpBitmap *bitmap, void **dest, GpImageAttri
 
 
 /* Bitmap */
+GpStatus GdipCreateBitmapFromFile(GDIPCONST WCHAR* filename, GpBitmap **bitmap);
+GpStatus GdipCreateBitmapFromFileICM(GDIPCONST WCHAR* filename, GpBitmap **bitmap);
 GpStatus GdipCreateBitmapFromScan0 (int width, int height, int strideIn, int format, void *scan0, GpBitmap **bitmap);
 GpStatus GdipCreateBitmapFromGraphics (int width, int height, GpGraphics *graphics, GpBitmap **bitmap);
 GpStatus GdipBitmapLockBits (GpBitmap *bmp, Rect *rc, int flags, int format, GdipBitmapData *result);
@@ -1248,23 +1293,28 @@ void gdip_calculate_coefficients (int count, int terms, float **coefficients, in
 
 /* Memory */
 void *GdipAlloc (int size);
+void *GdipCalloc (size_t nelem, size_t elsize); 
 void GdipFree (void *ptr);
 
 /* Utility*/
 int fcmp (double x1, double x2, double epsilon);
 float gdip_get_display_dpi();
 void gdip_unitConversion(Unit fromUnit, Unit toUnit, float nSrc, float* nTrg);
-
-void gdip_font_drawunderline (GpGraphics *graphics, GpBrush *brush, float x, float y, float width);
-void gdip_font_drawstrikeout (GpGraphics *graphics, GpBrush *brush, float x, float y, float width);
-
-cairo_status_t gdip_cairo_set_surface_pattern (cairo_t *t, cairo_surface_t *s);
+cairo_content_t from_cairoformat_to_content (cairo_format_t format);
 
 void gdip_rect_expand_by (GpRectF *rect, GpPointF *point);
 
 cairo_surface_t * gdip_bitmap_ensure_surface (GpBitmap *bitmap);
+BOOL gdip_is_an_alpha_pixelformat (PixelFormat pixfmt);
+BOOL gdip_is_an_indexed_pixelformat (PixelFormat pixfmt);
+GpBitmap * gdip_convert_indexed_to_rgb (GpBitmap *bitmap);
 
 const EncoderParameter *gdip_find_encoder_parameter (GDIPCONST EncoderParameters *eps, const GUID *guid);
+gchar *ucs2_to_utf8(const gunichar2 *ucs2, int length);
+bool utf8_to_ucs2(const gchar *utf8, gunichar2 *ucs2, int ucs2_len);
+int utf8_encode_ucs2char(gunichar2 unichar, unsigned char *dest);
+int utf8_decode_ucs2char(const unsigned char *src, gunichar2 *uchar);
+
 
 /* Stream handling bits */
 typedef int (*GetHeaderDelegate) (unsigned char *, int);
