@@ -28,6 +28,8 @@
 #include <cairo.h>
 #include <math.h>
 
+/* #define DEBUG_MEMLEAKS	1 */
+
 /* Startup / shutdown */
 
 struct startupInput
@@ -45,23 +47,35 @@ struct startupOutput
 	void *unhook;
 };
 
-extern void gdip_release_cachedfonts ();
 
+static GList* g_mem_allocations;
+
+extern void cairo_test_xlib_disable_render();
 
 GpStatus 
 GdiplusStartup(unsigned long *token, const struct startupInput *input, struct startupOutput *output)
 {
+	g_mem_allocations = NULL;
         initCodecList (); 
 	*token = 1;
 	gdip_get_display_dpi();
+//	cairo_test_xlib_disable_render();
 	return Ok;
 }
 
 void 
 GdiplusShutdown(unsigned long *token)
 {
+#ifdef DEBUG_MEMLEAKS
+	GList* list = NULL;
+#endif
 	releaseCodecList ();
-	gdip_release_cachedfonts ();
+
+#ifdef DEBUG_MEMLEAKS
+	for (list = g_list_first (g_mem_allocations); list != NULL; list = g_list_next (list)) {
+		printf ("Memory block not free'd at %x\n", list->data);
+	}
+#endif
 }
 
 
@@ -69,13 +83,47 @@ GdiplusShutdown(unsigned long *token)
 void *
 GdipAlloc (int size)
 {
+#ifdef DEBUG_MEMLEAKS		
+	void* block = malloc (size);
+	g_mem_allocations =  g_list_append (g_mem_allocations, block);
+	printf ("alloc %x %u\n", block, size);
+	return block;
+#else
 	return malloc (size);
+#endif
+}
+
+void *
+GdipCalloc (size_t nelem, size_t elsize)
+{
+#ifdef DEBUG_MEMLEAKS		
+	void* block = calloc (nelem, elsize);
+	g_mem_allocations =  g_list_append (g_mem_allocations, block);
+	printf ("calloc %x %u\n", block, nelem);
+	return block;
+#else
+	return calloc (nelem, elsize);
+#endif
+
 }
 
 void 
 GdipFree (void *ptr)
 {
+#ifdef DEBUG_MEMLEAKS	
+	GList* list = NULL;
+
+	list = g_list_find (g_list_first (g_mem_allocations), ptr);
+	if (list != NULL) {
+		list  = g_list_delete_link  (g_list_first (g_mem_allocations), list);
+		g_mem_allocations = g_list_last (list);
+	}
+
+	printf ("free %x\n", ptr);
 	free (ptr);
+#else
+	free (ptr);
+#endif
 }
 
 /* Helpers */
@@ -97,7 +145,6 @@ gdip_get_status (cairo_status_t status)
                         return InvalidParameter;                
 
                 case CAIRO_STATUS_NO_CURRENT_POINT:
-                case CAIRO_STATUS_NO_TARGET_SURFACE:
                         return WrongState;
 
                 default:
@@ -155,7 +202,7 @@ gdip_get_display_dpi()
 		char *val;
 
 		display = XOpenDisplay (0);
-		// If the display is openable lets try to read dpi from it; otherwise use a default of 96.0f
+		/* If the display is openable lets try to read dpi from it; otherwise use a default of 96.0f */
 		if (display) {
 			val = XGetDefault (display, "Xft", "dpi");
 			XCloseDisplay (display);
@@ -304,7 +351,7 @@ gdip_closed_curve_tangents (int terms, const GpPointF *points, int count, float
 
                 if (r >= count) r -= count;
                 if (s < 0) s += count;
-
+		
                 tangents [i].X += (coefficient * (points [r].X - points [s].X));
                 tangents [i].Y += (coefficient * (points [r].Y - points [s].Y));
         }
@@ -312,19 +359,6 @@ gdip_closed_curve_tangents (int terms, const GpPointF *points, int count, float
         return tangents;
 }
 
-cairo_status_t
-gdip_cairo_set_surface_pattern (cairo_t *t, cairo_surface_t *s)
-{
-    cairo_pattern_t *pat;
-    pat = cairo_pattern_create_for_surface (s);
-    if (pat == NULL)
-	return CAIRO_STATUS_NO_MEMORY;
-    cairo_set_pattern (t, pat);
-    cairo_pattern_destroy (pat);
-
-    return cairo_status (t);
-}
-
 void
 gdip_rect_expand_by (GpRectF *rect, GpPointF *point)
 {
@@ -422,3 +456,140 @@ float gdip_erf (float x, float std, float mean)
 
 	return constant * series;
 }
+
+/*
+ convert a ucs2 string to utf8
+ length = number of characters to convert, -1 to indicate the whole string
+*/
+
+gchar *
+ucs2_to_utf8(const gunichar2 *ucs2, int length) {
+	const gunichar2	*ptr;
+	const gunichar2	*end;
+	gunichar	*dest;
+	gunichar	*uni;
+	gchar		*utf8;
+
+	/* Count length */
+	if (length == -1) {
+		ptr = ucs2;
+		length = 0;
+		while (*ptr != 0) {
+			ptr++;
+			length++;
+		}
+	}
+
+
+	uni = GdipAlloc((length + 1) * sizeof(gunichar));
+	if (uni == NULL) {
+		return NULL;
+	}
+
+	dest = uni;
+	ptr = ucs2;
+	end = ptr + length;
+	while (ptr != end) {
+		if (*ptr < 0xd800 || *ptr >= 0xe000) {
+			*dest = *ptr;
+			dest++;
+		}
+		ptr++;
+	}
+	*dest = 0;
+	dest++;
+	
+	utf8 = (gchar *) g_ucs4_to_utf8 ((const gunichar *)uni, -1, NULL, NULL, NULL);
+
+	GdipFree(uni);
+
+	return utf8;
+}
+
+bool
+utf8_to_ucs2(const gchar *utf8, gunichar2 *ucs2, int ucs2_len) {
+	int 		i;
+	glong		items_read;
+	glong		count;
+	gunichar	*ucs4;
+	unsigned char	*ptr;
+
+	items_read = 0;
+	count = 0;
+
+	ucs2_len--;	/* Space for null terminator */
+
+	ucs4 = g_utf8_to_ucs4(utf8, -1, &items_read, &count, NULL);
+	if (ucs4 == NULL) {
+		return FALSE;
+	}
+
+	ptr = (unsigned char *)ucs2;
+	for (i = 0; (i < count) && (i < ucs2_len); i++) {
+		if (ucs4[i] < 0x1000 && !(ucs4[i] >= 0xd800 && ucs4[i] < 0xe000)) {
+			ptr[0] = (unsigned char)ucs4[i];
+			ptr[1] = (unsigned char)(ucs4[i] >> 8);
+			ptr += 2;
+		}	/* we're simply ignoring any chars that don't fit into ucs2 */
+	}
+	ucs2[i] = 0;	/* terminate */
+
+	/* free the intermediate ucs4 string */
+	GdipFree(ucs4);
+
+	return TRUE;
+}
+
+int
+utf8_encode_ucs2char(gunichar2 unichar, unsigned char *dest)
+{
+	if (unichar < 0x0080) {					/* 0000-007F */
+		dest[0] = (unsigned char)(unichar);
+		return (1);
+	}
+	if(unichar < 0x0800) {					/* 0080-07FF */
+		dest[0] = (unsigned char)(0xC0 | ((unichar & 0x07C0) >> 6));
+		dest[1] = (unsigned char)(0x80 | (unichar & 0x003F));
+		return (2);
+	}
+								/* 0800-FFFF */
+	dest[0] = (unsigned char)(0xE0 | ((unichar & 0xF000) >> 12));
+	dest[1] = (unsigned char)(0x80 | ((unichar & 0x0FC0) >> 6));
+	dest[2] = (unsigned char)(0x80 | (unichar & 0x003F));
+	return (3);	
+}
+
+/* This function only handles UCS-2 */
+int
+utf8_decode_ucs2char(const unsigned char *src, gunichar2 *uchar)
+{
+	if (src[0] <= 0x7F) {			/* 0000-007F: one byte (0xxxxxxx) */
+		*uchar = (gunichar2)src[0];
+		return (1);
+	}
+	if (src[0] <= 0xDF) {			/* 0080-07FF: two bytes (110xxxxx 10xxxxxx) */
+		*uchar = ((((gunichar2)src[0]) & 0x001F) << 6) |
+			((((gunichar2)src[1]) & 0x003F) << 0);
+		return (2);
+	}
+						/* 0800-FFFF: three bytes (1110xxxx 10xxxxxx 10xxxxxx) */
+	*uchar = ((((gunichar2)src[0]) & 0x000F) << 12) |
+		((((gunichar2)src[1]) & 0x003F) << 6) |
+		((((gunichar2)src[2]) & 0x003F) << 0);
+	return (3);
+}
+
+cairo_content_t
+from_cairoformat_to_content (cairo_format_t format)
+{
+    	switch (format) {
+    	case CAIRO_FORMAT_RGB24:
+		return CAIRO_CONTENT_COLOR;
+	case CAIRO_FORMAT_A8:
+		return CAIRO_CONTENT_ALPHA;
+	case CAIRO_FORMAT_ARGB32:
+	default:
+		return CAIRO_CONTENT_COLOR_ALPHA;
+    }
+}
+
