@@ -560,7 +560,6 @@ append_arc (GpPath *path, bool start, float x, float y, float width, float heigh
         /* angles in radians */        
         float alpha = startAngle * PI / 180;
         float beta = endAngle * PI / 180;
-
         float delta = beta - alpha;
         float bcp = 4.0 / 3 * (1 - cos (delta / 2)) / sin (delta / 2);
 
@@ -572,10 +571,10 @@ append_arc (GpPath *path, bool start, float x, float y, float width, float heigh
         /* starting point */
         double sx = cx + rx * cos_alpha;
         double sy = cy + ry * sin_alpha;
-
+	
         /* move to the starting point if we're not continuing a curve */
         if (start)
-                append (path, sx, sy, PathPointTypeStart);
+                append (path, sx, sy, PathPointTypeLine);
 
         append_bezier (path, 
                 cx + rx * (cos_alpha - bcp * sin_alpha),
@@ -586,21 +585,50 @@ append_arc (GpPath *path, bool start, float x, float y, float width, float heigh
                 cy + ry *  sin_beta);
 }
 
+static void
+append_arcs (GpPath *path, float x, float y, float width, float height, float startAngle, float sweepAngle)
+{
+	int i;
+	float drawn = 0;
+	float endAngle = startAngle + sweepAngle;	
+	int sign = (endAngle > 0) ? 1 : -1;
+	int increment = sign * 90; 
+	
+	if (abs (sweepAngle) >= 360) {
+		GdipAddPathEllipse (path, x, y, width, height);
+		return;
+	}
+
+	/* i is the number of sub-arcs drawn, each sub-arc can be at most 90 degrees.*/
+	/* there can be no more then 4 subarcs, ie. 90 + 90 + 90 + (something less than 90) */
+	for (i = 0; i < 4; i++) {
+		float current = startAngle + drawn;
+
+		/* we've drawn enough */
+		if (abs (current) >= abs (endAngle))
+			break;
+		
+		/* if we are not done yet */
+		float additional = abs (current + increment) < abs (endAngle) ?
+				increment : /* add the default increment */
+				endAngle - current; /* otherwise, add the remainder */
+		
+		append_arc (path,
+			    (i == 0) ? TRUE : FALSE,  /* only move to the starting pt in the 1st iteration */
+			    x, y, width, height,      /* bounding rectangle */
+			    current, current + additional);
+		drawn += additional;		
+	}
+}
+
 GpStatus
 GdipAddPathArc (GpPath *path, float x, float y, 
                 float width, float height, float startAngle, float sweepAngle)
 {
-        float endAngle = startAngle + sweepAngle;
 	g_return_val_if_fail (path != NULL, InvalidParameter);
-        
-        if (sweepAngle < 180)
-                append_arc (path, TRUE, x, y, width, height, startAngle, endAngle);
-        else {
-                float midAngle = startAngle + (sweepAngle / 2.0);
 
-                append_arc (path, TRUE, x, y, width, height, startAngle, midAngle);
-                append_arc (path, FALSE, x, y, width, height, midAngle, endAngle);
-        }
+	/* draw the arcs */
+	append_arcs (path, x, y, width, height, startAngle, sweepAngle);
 
         return Ok;
 }
@@ -769,10 +797,7 @@ GdipAddPathPie (GpPath *path, float x, float y, float width, float height, float
 
         /* center */
         int cx = x + rx;
-        int cy = y + ry;
-
-        float endAngle = startAngle + sweepAngle;        
-
+        int cy = y + ry;	
         /* angles in radians */        
         float alpha = startAngle * PI / 180;
 
@@ -783,22 +808,13 @@ GdipAddPathPie (GpPath *path, float x, float y, float width, float height, float
 
         /* move to center */
         append (path, cx, cy, PathPointTypeStart);
-        
 
         /* draw pie edge */
         append (path, cx + rx * cos_alpha, cy + ry * sin_alpha,
                 PathPointTypeLine);
 
-        /* draw arc */
-        if (sweepAngle < 180)
-                append_arc (path, TRUE, x, y, width, height, startAngle, endAngle);
-
-        else {
-                float midAngle = startAngle + (sweepAngle / 2.0);
-
-                append_arc (path, TRUE, x, y, width, height, startAngle, midAngle);
-                append_arc (path, FALSE, x, y, width, height, midAngle, endAngle);
-        }
+	/* draw the arcs */
+	append_arcs (path, x, y, width, height, startAngle, sweepAngle);
         
         /* draw pie edge */
         append (path, cx, cy, PathPointTypeLine);
@@ -872,8 +888,8 @@ GdipAddPathPath (GpPath *path, GpPath *addingPath, bool connect)
 	for (i = 1; i < length; i++)
 		append_point (path, pts [i], types [i]);
 
-	free(pts);
-	free(types);
+	GdipFree(pts);
+	GdipFree(types);
 
 	return Ok;
 }
