@@ -24,16 +24,15 @@
  *   Sanjay Gupta (gsanjay@novell.com)
  *   Vladimir Vukicevic (vladimir@pobox.com)
  *   Jordi Mas (jordi@ximian.com)
+ *   Jonathan Gilbert (logic@deltaq.org)
  */
 
 #include <stdio.h>
 #include "gdipImage.h"
 #include <math.h>
 
-
-static char *guid_to_string_hack (GDIPCONST CLSID *clsid);
-void gdip_FlipX (GpImage *image);
-void gdip_rotate_180_FlipX (GpImage *image);
+static GpStatus gdip_flip_x (GpImage *image);
+static GpStatus gdip_flip_y (GpImage *image);
 cairo_filter_t gdip_get_cairo_filter (InterpolationMode imode);
 
 /*
@@ -94,7 +93,7 @@ GdipDisposeImage (GpImage *image)
 			dataCount = image->frameDimensionList [i].count;
 			data = image->frameDimensionList [i].frames;
 				for (j = 0; j < dataCount; j++) {
-					if (data [j].Scan0) {
+					if (((data [j].Reserved & GBD_OWN_SCAN0) != 0) && data [j].Scan0 != NULL) {
 						GdipFree (data [j].Scan0);
 						data [j].Scan0 = NULL;
 					}
@@ -125,18 +124,24 @@ GpStatus
 GdipGetImageGraphicsContext (GpImage *image, GpGraphics **graphics)
 {
 	GpGraphics *gfx;
+	cairo_surface_t *surface;
+	GpBitmap* bmp = (GpBitmap *) image;
+	
 	if (!image || !graphics)
 		return InvalidParameter;
 
-	gfx = gdip_graphics_new ();
-	if (image->type == imageBitmap) {
-		gdip_graphics_attach_bitmap (gfx, (GpBitmap *) image);
-	}
-	else if (image->type == imageMetafile) {
-	}
+	if (image->type != imageBitmap)
+		return NotImplemented;	
+	
+	surface = cairo_image_surface_create_for_data ((unsigned char *) bmp->data.Scan0, bmp->cairo_format,
+				bmp->data.Width, bmp->data.Height, bmp->data.Stride);
 
+	gfx = gdip_graphics_new (surface);
+
+	gfx->image = bmp;
+	gfx->type = gtMemoryBitmap;	
+	cairo_pattern_set_filter (cairo_pattern_create_for_surface (bmp->image.surface), gdip_get_cairo_filter (gfx->interpolation));
 	*graphics = gfx;
-
 	return Ok;
 }
 
@@ -161,15 +166,35 @@ GdipDrawImageRectI (GpGraphics *graphics, GpImage *image, int x, int y, int widt
 GpStatus
 GdipDrawImageRect (GpGraphics *graphics, GpImage *image, float x, float y, float width, float height)
 {
+	cairo_pattern_t *pattern;
+	
 	g_return_val_if_fail (graphics != NULL, InvalidParameter);
 	g_return_val_if_fail (image != NULL, InvalidParameter);
 	g_return_val_if_fail (image->type == imageBitmap, InvalidParameter);
 	
 	cairo_new_path(graphics->ct);
 
-	/* Create a surface for this bitmap if one doesn't exist */    	
-	gdip_bitmap_ensure_surface ((GpBitmap*) image);
-	cairo_surface_set_filter (((GpBitmap*) image)->image.surface, gdip_get_cairo_filter (graphics->interpolation));
+	if (gdip_is_an_indexed_pixelformat (((GpBitmap*) image)->data.PixelFormat)) {
+		/* Unable to create a surface for the bitmap; it is an indexed image.
+		 * Instead, it will first be converted to 32-bit RGB.
+		 */
+		GpStatus status = OutOfMemory;
+
+		GpBitmap *rgb_bitmap = gdip_convert_indexed_to_rgb ((GpBitmap *) image);
+
+		if (rgb_bitmap != NULL) {
+			status = GdipDrawImageRect(graphics, (GpImage *)rgb_bitmap, x, y, width, height);
+			GdipDisposeImage((GpImage *)rgb_bitmap);
+		}
+
+		return status;
+	}
+
+	/* Create a surface for this bitmap if one doesn't exist */
+	gdip_bitmap_ensure_surface ((GpBitmap *) image);
+	pattern = cairo_pattern_create_for_surface (((GpBitmap*) image)->image.surface);
+	cairo_pattern_set_filter (pattern,
+				  gdip_get_cairo_filter (graphics->interpolation));
 
 	cairo_translate (graphics->ct, x, y);
 
@@ -179,9 +204,12 @@ GdipDrawImageRect (GpGraphics *graphics, GpImage *image, float x, float y, float
 				(double) height / image->height);
 	}
 	
-	cairo_show_surface (graphics->ct, image->surface, image->width, image->height);
-	cairo_default_matrix (graphics->ct);
-
+	cairo_set_source_surface (graphics->ct, image->surface, 0, 0);
+	cairo_identity_matrix (graphics->ct);
+	cairo_paint (graphics->ct);
+	
+	cairo_pattern_destroy (pattern);
+	
 	return Ok;
 }
 
@@ -189,19 +217,56 @@ GdipDrawImageRect (GpGraphics *graphics, GpImage *image, float x, float y, float
 GpStatus
 GdipDrawImagePoints (GpGraphics *graphics, GpImage *image, GDIPCONST GpPointF *dstPoints, int count)
 {
-	float x, y, width, height;
+	float width, height;
+	cairo_pattern_t *pattern;
 	
 	g_return_val_if_fail (graphics != NULL, InvalidParameter);
 	g_return_val_if_fail (image != NULL, InvalidParameter);
 	g_return_val_if_fail (dstPoints != NULL, InvalidParameter);
-	g_return_val_if_fail (count != 3, InvalidParameter);
+	g_return_val_if_fail (count == 3, InvalidParameter);
+
+	if (gdip_is_an_indexed_pixelformat (((GpBitmap*) image)->data.PixelFormat)) {
+		/* Unable to create a surface for the bitmap; it is an indexed image.
+		 * Instead, it will first be converted to 32-bit RGB.
+		 */
+		GpStatus status = OutOfMemory;
+
+		GpBitmap *rgb_bitmap = gdip_convert_indexed_to_rgb ((GpBitmap *) image);
+
+		if (rgb_bitmap != NULL) {
+			status = GdipDrawImagePoints (graphics, (GpImage *)rgb_bitmap, dstPoints, count);
+			GdipDisposeImage((GpImage *)rgb_bitmap);
+		}
+		return status;
+	}
+
+	cairo_new_path(graphics->ct);
+
+	width = dstPoints[1].X > dstPoints[0].X ? dstPoints[1].X - dstPoints[0].X : dstPoints[0].X - dstPoints[1].X;
+	height = dstPoints[2].Y > dstPoints[0].Y ? dstPoints[2].Y - dstPoints[0].Y : dstPoints[0].Y - dstPoints[2].Y;
+
+	/* Create a surface for this bitmap if one doesn't exist */
+	gdip_bitmap_ensure_surface ((GpBitmap *) image);
+	pattern = cairo_pattern_create_for_surface (((GpBitmap*) image)->image.surface);
+	cairo_pattern_set_filter (pattern,
+				  gdip_get_cairo_filter (graphics->interpolation));
+
+	cairo_translate (graphics->ct, dstPoints[0].X, dstPoints[0].Y);
+
+	/*
+		TODO: Implement rotation
+	*/
+
+	cairo_scale (graphics->ct,
+		(double) width / image->width,
+		(double) height / image->height);
 	
-	x = dstPoints[0].X;
-	y = dstPoints[0].Y;
-	width = dstPoints[1].X - dstPoints[0].X;
-	height = dstPoints[2].Y - dstPoints[2].Y;
+	cairo_set_source_surface (graphics->ct, image->surface, 0, 0);
+	cairo_identity_matrix (graphics->ct);
+	cairo_paint (graphics->ct);	
+	cairo_pattern_destroy (pattern);
 	
-	return GdipDrawImageRect (graphics, image, x, y, width, height);
+	return Ok;
 }
 
 GpStatus
@@ -211,11 +276,11 @@ GdipDrawImagePointsI (GpGraphics *graphics, GpImage *image, GDIPCONST GpPoint *d
 	int i;
 	
 	g_return_val_if_fail (dstPoints != NULL, InvalidParameter);
-	g_return_val_if_fail (count != 3, InvalidParameter);
+	g_return_val_if_fail (count == 3, InvalidParameter);
 	
 	for (i = 0; i < 3; i++) {
-	points[i].X = dstPoints[i].X;
-	points[i].Y = dstPoints[i].Y;
+		points[i].X = dstPoints[i].X;
+		points[i].Y = dstPoints[i].Y;
 	}
 	
 	return GdipDrawImagePoints (graphics, image, points, 3);
@@ -248,15 +313,36 @@ GdipDrawImageRectRect (GpGraphics *graphics, GpImage *image,
                        DrawImageAbort callback, void *callbackData)
 {
 	cairo_pattern_t *pattern;
-	cairo_matrix_t *mat = NULL;
+	cairo_matrix_t mat;
 	void* dest, *org;
 	bool allocated = FALSE;
 	GpBitmap *bitmap = (GpBitmap *) image;
 	
+	cairo_matrix_init (&mat, 1, 0, 0, 1, 0, 0);
+
 	g_return_val_if_fail (graphics != NULL, InvalidParameter);
 	g_return_val_if_fail (image != NULL, InvalidParameter);
 	g_return_val_if_fail (image->type == imageBitmap, InvalidParameter);
 
+	if (gdip_is_an_indexed_pixelformat (((GpBitmap *)image)->data.PixelFormat)) {
+                /* Unable to create a surface for the bitmap; it is an indexed image.
+                 * Instead, it will first be converted to 32-bit RGB.
+                 */
+		GpStatus status = OutOfMemory;
+
+		GpBitmap *rgb_bitmap = gdip_convert_indexed_to_rgb ((GpBitmap *)image);
+
+		if (rgb_bitmap != NULL) {
+			status = GdipDrawImageRectRect (graphics, (GpImage *)rgb_bitmap,
+				dstx, dsty, dstwidth, dstheight,
+				srcx, srcy, srcwidth, srcheight,
+				srcUnit, imageAttributes, callback, callbackData);
+			GdipDisposeImage ((GpImage *)rgb_bitmap);
+		}
+
+		return status;
+	}
+
 	if (srcUnit != UnitPixel && srcUnit != UnitWorld) {
 		gdip_unitConversion(srcUnit, UnitPixel, dstx, &dstx);
 		gdip_unitConversion(srcUnit, UnitPixel, dsty, &dsty);
@@ -284,7 +370,6 @@ GdipDrawImageRectRect (GpGraphics *graphics, GpImage *image,
 	}
 
 	if (imageAttributes && imageAttributes->wrapmode != WrapModeClamp) {
-		float img_x = srcx - dstx, img_y = srcy - dsty;
 		float img_width = bitmap->data.Width *  (dstwidth / srcwidth) , img_height = bitmap->data.Height * (dstheight / srcheight);
 		float posx, posy;
 		bool flipXOn = (imageAttributes->wrapmode == WrapModeTileFlipX) ? TRUE: FALSE;
@@ -297,26 +382,26 @@ GdipDrawImageRectRect (GpGraphics *graphics, GpImage *image,
 		
 		if (flipXOn) {			
 			gdip_bitmap_clone (bitmap, &imgflipX);
-			gdip_FlipX ((GpImage *) imgflipX);	
+			gdip_flip_x ((GpImage *) imgflipX);	
 			gdip_bitmap_ensure_surface (imgflipX);			
 		}
 		
 		if (flipYOn) {			
 			gdip_bitmap_clone (bitmap, &imgflipY);
-			gdip_rotate_180_FlipX ((GpImage *) imgflipY);	
+			gdip_flip_y ((GpImage *) imgflipY);	
 			gdip_bitmap_ensure_surface (imgflipY);			
 		}
 		
 		if (flipXOn && flipYOn) {			
 			gdip_bitmap_clone (bitmap, &imgflipXY);
-			gdip_FlipX ((GpImage *)imgflipXY);	
-			gdip_rotate_180_FlipX ((GpImage *) imgflipXY);	
+			gdip_flip_x ((GpImage *)imgflipXY);	
+			gdip_flip_y ((GpImage *) imgflipXY);	
 			gdip_bitmap_ensure_surface (imgflipXY);			
 		}
 		
 		/* Create a surface for this bitmap if one doesn't exist */
 		gdip_bitmap_ensure_surface ((GpBitmap*) image);
-		cairo_surface_set_filter (((GpBitmap*) image)->image.surface, gdip_get_cairo_filter (graphics->interpolation));
+		cairo_pattern_set_filter (cairo_pattern_create_for_surface(((GpBitmap*) image)->image.surface), gdip_get_cairo_filter (graphics->interpolation));
 
 		for (posy = 0; posy < dstheight; posy += img_height) {
 		
@@ -333,18 +418,21 @@ GdipDrawImageRectRect (GpGraphics *graphics, GpImage *image,
 						else 
 							cur_image = bitmap;
 				
-				mat = cairo_matrix_create ();
-				cairo_matrix_scale (mat, srcwidth / dstwidth, srcheight / dstheight);
-				cairo_matrix_translate (mat, srcx - (dstx + posx), srcy - (dsty + posy));
-				cairo_surface_set_matrix (cur_image->image.surface, mat);	
-				gdip_cairo_set_surface_pattern (graphics->ct, cur_image->image.surface);
+
+				cairo_matrix_scale (&mat, srcwidth / dstwidth, srcheight / dstheight);
+				cairo_matrix_translate (&mat, srcx - (dstx + posx), srcy - (dsty + posy));
+
+				pattern = cairo_pattern_create_for_surface(cur_image->image.surface);
+				cairo_pattern_set_matrix (pattern, &mat);
+				cairo_set_source(graphics->ct, pattern);
 				
 				cairo_rectangle (graphics->ct, dstx + posx, dsty + posy, img_width, img_height);
 				cairo_fill (graphics->ct);
 				
-				cairo_matrix_set_identity (mat);
-				cairo_surface_set_matrix (cur_image->image.surface, mat);
-				cairo_matrix_destroy (mat);			
+				cairo_matrix_init_identity (&mat);
+				cairo_pattern_set_matrix (pattern, &mat);
+
+				cairo_pattern_destroy(pattern);
 				
 				if (flipXOn)
 					flipX = !flipX; 					
@@ -366,26 +454,23 @@ GdipDrawImageRectRect (GpGraphics *graphics, GpImage *image,
 		
 	}
 	else  {		
-		
 		/* Create a surface for this bitmap if one doesn't exist */
 		gdip_bitmap_ensure_surface ((GpBitmap*) image);
-		cairo_surface_set_filter (((GpBitmap*) image)->image.surface, gdip_get_cairo_filter (graphics->interpolation));
+		cairo_pattern_set_filter (cairo_pattern_create_for_surface (((GpBitmap*) image)->image.surface) , gdip_get_cairo_filter (graphics->interpolation));
 		
-		mat = cairo_matrix_create ();
-		cairo_matrix_scale (mat, srcwidth / dstwidth, srcheight / dstheight);
-		cairo_matrix_translate (mat, srcx - dstx, srcy - dsty);
-		cairo_surface_set_matrix (image->surface, mat);
-		
-		gdip_cairo_set_surface_pattern (graphics->ct, image->surface);
+		cairo_matrix_scale (&mat, srcwidth / dstwidth, srcheight / dstheight);
+		cairo_matrix_translate (&mat, srcx - dstx, srcy - dsty);
+
+		pattern = cairo_pattern_create_for_surface(image->surface);
+		cairo_pattern_set_matrix (pattern, &mat);
+		cairo_set_source(graphics->ct, pattern);
 		
 		cairo_rectangle (graphics->ct, dstx, dsty, dstwidth, dstheight);
 		cairo_fill (graphics->ct);
 		
-		cairo_matrix_set_identity (mat);
-		cairo_surface_set_matrix (image->surface, mat);
-		
-		/* unref */
-		cairo_matrix_destroy (mat);
+		cairo_matrix_init_identity (&mat);
+		cairo_pattern_set_matrix (pattern, &mat);
+		cairo_pattern_destroy (pattern);
 	}
 
 	/* The default surface is no longer valid*/
@@ -396,7 +481,7 @@ GdipDrawImageRectRect (GpGraphics *graphics, GpImage *image,
 	
 	if (allocated) {
 		bitmap->data.Scan0 = org;
-		free (dest);	
+		GdipFree (dest);	
 	}
 	
 	return Ok;
@@ -464,21 +549,19 @@ GdipLoadImageFromFile (GDIPCONST WCHAR *file, GpImage **image)
 	char format_peek[10];
 	int format_peek_sz;
 	
-	GError *err = NULL;
-	
 	if (!image || !file)
 		return InvalidParameter;
 	
-	file_name = (char *) g_utf16_to_utf8 ((const gunichar2 *)file, -1, NULL, NULL, &err);
-	if (file_name == NULL || err != NULL) {
+	file_name = (char *) ucs2_to_utf8 ((const gunichar2 *)file, -1);
+	if (file_name == NULL) {
 		*image = NULL;
 		return InvalidParameter;
-		}
+	}
 	
 	fp = fopen(file_name, "rb");
-	g_free (file_name);
+	GdipFree (file_name);
 	if (fp == NULL)
-		return FileNotFound;
+		return InvalidParameter;
 	
 	format_peek_sz = fread (format_peek, 1, 10, fp);
 	format = get_image_format (format_peek, format_peek_sz);
@@ -547,18 +630,18 @@ gdip_get_imageformat_from_codec_clsid (CLSID *encoderCLSID)
     	if (numEncoders == 0)
         	return INVALID;
 
-    	encoders = malloc (size);
+    	encoders = GdipAlloc (size);
     
     	GdipGetImageEncoders (numEncoders, size, encoders);
 
     	for (cnt = 0, encoder = encoders; cnt < numEncoders; encoder++) {
 	       	if (memcmp (&encoder->Clsid, encoderCLSID, sizeof (GUID)) == 0) {
-            		free (encoders);
+            		GdipFree (encoders);
             		return gdip_image_format_for_format_guid (&encoder->FormatID);
         	}
     	}
 
-    	free (encoders);
+    	GdipFree (encoders);
     	return INVALID;
 }
 
@@ -568,7 +651,6 @@ GdipSaveImageToFile (GpImage *image, GDIPCONST WCHAR *file, GDIPCONST CLSID *enc
 	FILE *fp = 0;
 	GpStatus status = 0;
 	char *file_name;
-	GError *err = NULL;
 	ImageFormat format;
 	
 	if (image->type != imageBitmap)
@@ -581,20 +663,20 @@ GdipSaveImageToFile (GpImage *image, GDIPCONST WCHAR *file, GDIPCONST CLSID *enc
 	if (format == INVALID)
 		return UnknownImageFormat;
 	
-	file_name = (char *) g_utf16_to_utf8 ((const gunichar2 *)file, -1, NULL, NULL, &err);
-	if (file_name == NULL || err != NULL) 
+	file_name = (char *) ucs2_to_utf8 ((const gunichar2 *)file, -1);
+	if (file_name == NULL)
 		return InvalidParameter;
 	
 	if (format == GIF) { /* gif library has to open the file itself*/
-		status = gdip_save_gif_image_to_file (file_name, image);
-		g_free (file_name);
+		status = gdip_save_gif_image_to_file ((unsigned char *)file_name, image);
+		GdipFree (file_name);
 		return status;
 	}
 	
 	if ((fp = fopen(file_name, "wb")) == NULL)
 		return GenericError;
 		
-	g_free (file_name);
+	GdipFree (file_name);
 	
 	switch (format) {
 		case BMP:
@@ -708,8 +790,6 @@ GdipGetImageFlags (GpImage *image, UINT *flags)
 
 GpStatus GdipGetImageRawFormat (GpImage *image, GUID *format)
 {
-	byte* guid;
-
 	if (!image || !format)
 		return InvalidParameter;
 	
@@ -827,13 +907,42 @@ GdipImageSelectActiveFrame(GpImage *image, GDIPCONST GUID *dimensionGUID, UINT i
         return InvalidParameter;        
 }
 
-void copy_pixel (BYTE *src, BYTE *trg, int components)
+void copy_pixel (BYTE *src, BYTE *trg, int size)
 {
-	memcpy (trg, src, components);
+	memcpy (trg, src, size);
 }
 
-void
-gdip_rotate_180_FlipX (GpImage *image)
+static GpStatus
+gdip_flip_x (GpImage *image)
+{
+	BYTE *src, *line;
+	int stride, width, height, pixel_size, i, j;
+	GpBitmap *bitmap = (GpBitmap *) image;
+	
+	stride = bitmap->data.Stride;
+	width = bitmap->data.Width;
+	height = bitmap->data.Height;
+	pixel_size = gdip_get_pixel_format_components (bitmap->data.PixelFormat) * gdip_get_pixel_format_depth (bitmap->data.PixelFormat) / 8;
+	line = GdipAlloc (stride);
+	src = (BYTE *) bitmap->data.Scan0;
+
+	if (line == NULL)
+		return OutOfMemory;
+	
+	for (i = 0; i < height; i++, src += stride) {			
+		memcpy (line, src, stride);	/* Save original line*/
+
+		for  (j = 0; j < width; j++)
+			copy_pixel(&line[(width - j - 1) * pixel_size], &src[j * pixel_size], pixel_size);
+	}
+	
+	GdipFree (line);
+
+	return Ok;
+}
+
+static GpStatus
+gdip_flip_y (GpImage *image)
 {
 	BYTE *src, *trg, *line;
 	int stride, height, i;	
@@ -841,10 +950,13 @@ gdip_rotate_180_FlipX (GpImage *image)
 	
 	stride = bitmap->data.Stride;
 	height = bitmap->data.Height;
-	line = malloc (stride);		
+	line = GdipAlloc (stride);		
 	src = (BYTE *) bitmap->data.Scan0;
 	trg = (BYTE *) bitmap->data.Scan0;
 	trg +=  (height-1) * stride;
+
+	if (line == NULL)
+		return OutOfMemory;
 	
 	for (i = 0; i < (height /2); i++, src += stride, trg -= stride) {			
 		memcpy (line, trg, stride);	/* Save target line*/		
@@ -852,237 +964,437 @@ gdip_rotate_180_FlipX (GpImage *image)
 		memcpy (src, line, stride);	/* Copy trg to src*/		
 	}
 	
-	free (line);
+	GdipFree (line);
+
+	return Ok;
 }
 
-void
-gdip_rotate_90 (GpImage *image)
+static GpStatus
+gdip_rotate_orthogonal_flip_x (GpImage *image, int angle, BOOL flip_x)
 {
-	BYTE *src, *trg, *rotated;
-	int line, x, stride_trg, height_trg, width_trg;
 	GpBitmap *bitmap = (GpBitmap *) image;
-	int components = gdip_get_pixel_format_components (bitmap->data.PixelFormat);		         	 
 
-	height_trg = bitmap->data.Width;
-	width_trg = bitmap->data.Height;
-	stride_trg = components * gdip_get_pixel_format_depth (bitmap->data.PixelFormat);
-	stride_trg = (stride_trg * width_trg) / 8;
-	stride_trg = (stride_trg + 3) & ~3;		
-	
-	rotated = malloc (stride_trg * height_trg);
-	src = (BYTE *) bitmap->data.Scan0;
-	
-	for (line = 0; line < bitmap->data.Height; line++, src += bitmap->data.Stride) {		
-		for (x = 0; x < bitmap->data.Width; x++) {
+	BYTE *rotated;
 
-			if (x >= height_trg)
-				continue;			
+	BYTE *source, *target;
+	int x, y;
+	int source_stride, source_height, source_width, source_pixel_delta, source_interscan_delta;
+	int target_stride, target_height, target_width, target_pixel_delta, target_interscan_delta;
+	int initial_source_offset, initial_target_offset;
 
-			if (line >= width_trg)
-				continue;
+	int components = gdip_get_pixel_format_components (bitmap->data.PixelFormat);
+	int depth = gdip_get_pixel_format_depth (bitmap->data.PixelFormat);
 
-			trg = rotated;
-			trg += x * stride_trg;
-			trg += (width_trg - 1  - line) * components;		
-			copy_pixel (src + (x * components), trg, components);			
-		}	
+	int pixel_size = components * depth / 8;
+
+	source_stride = bitmap->data.Stride;
+	source_width = bitmap->data.Width;
+	source_height = bitmap->data.Height;
+
+	source_pixel_delta = pixel_size;
+	source_interscan_delta = source_stride - source_width * pixel_size;
+	initial_source_offset = 0;
+
+	if (angle == 180) {
+		target_width = source_width;
+		target_height = source_height;
+	}
+	else {
+		target_height = source_width; /* swap width & height here */
+		target_width = source_height;
 	}
 
-	bitmap->data.Stride = stride_trg;
-	bitmap->data.Height = image->height = height_trg;
-	bitmap->data.Width = image->width = width_trg;
+	target_stride = target_width * pixel_size;
+	target_stride = (target_stride + 3) & ~3;
+
+	switch (angle)
+	{
+		case 90:
+			target_pixel_delta = target_stride;
+			if (flip_x) {
+				target_interscan_delta = +pixel_size - target_stride * target_height;
+				initial_target_offset = 0;
+			}
+			else {
+				target_interscan_delta = -pixel_size - target_stride * target_height;
+				initial_target_offset = (target_width - 1) * pixel_size;
+			}
+			break;
+		case 180:
+			if (flip_x) {
+				target_pixel_delta = +pixel_size;
+				target_interscan_delta = -target_width * pixel_size - target_stride; 
+				initial_target_offset = (target_height - 1) * target_stride;
+			}
+			else {
+				target_pixel_delta = -pixel_size;
+				target_interscan_delta = +target_width * pixel_size - target_stride;
+				initial_target_offset = (target_height - 1) * target_stride + (target_width - 1) * pixel_size;
+			}
+			break;
+		case 270:
+			target_pixel_delta = -target_stride;
+			if (flip_x) {
+				target_interscan_delta = target_stride * target_height - pixel_size;
+				initial_target_offset = (target_height - 1) * target_stride + (target_width - 1) * pixel_size;
+			}
+			else {
+				target_interscan_delta = target_stride * target_height + pixel_size;
+				initial_target_offset = (target_height - 1) * target_stride;
+			}
+			break;
+		default:
+			if (flip_x)
+				return gdip_flip_x (image);
+			else
+				return Ok;
+	}
+
+	rotated = GdipAlloc (target_height * target_stride);
+
+	if (rotated == NULL)
+		return OutOfMemory;
+
+	source = initial_source_offset + (BYTE *) bitmap->data.Scan0;
+	target = initial_target_offset + rotated;
+
+	for (y = 0; y < source_height;
+             y++,
+             source += source_interscan_delta,
+             target += target_interscan_delta)
+		for (x = 0; x < source_width;
+                     x++,
+                     source += source_pixel_delta,
+                     target += target_pixel_delta)
+			copy_pixel (source, target, pixel_size);
+
+	bitmap->data.Stride = target_stride;
+	bitmap->data.Height = image->height = target_height;
+	bitmap->data.Width = image->width = target_width;
 
 	if ((bitmap->data.Reserved & GBD_OWN_SCAN0) != 0) {
-		int k,j; 
+		int k, j; 
+
+		void *old_scan0 = bitmap->data.Scan0;
 
-		/* Do the free first since the frame point can be for some codes a reference to bitmap->data*/
-		free (bitmap->data.Scan0);
 		for (j = 0; j < image->frameDimensionCount; j++) {
 			for (k = 0; k < image->frameDimensionList [j].count; k++) {
-				if (image->frameDimensionList[j].frames[k].Scan0 == bitmap->data.Scan0)  {
+				if (image->frameDimensionList[j].frames[k].Scan0 == old_scan0)  {
 					image->frameDimensionList[j].frames[k].Scan0 = rotated;
-					image->frameDimensionList[j].frames[k].Stride = stride_trg;
-					image->frameDimensionList[j].frames[k].Height = image->height = height_trg;
-					image->frameDimensionList[j].frames[k].Width = image->width = width_trg;
-				}				
+					image->frameDimensionList[j].frames[k].Stride = target_stride;
+					image->frameDimensionList[j].frames[k].Height = image->height = target_height;
+					image->frameDimensionList[j].frames[k].Width = image->width = target_width;
+				}
 			}
-		}		
+		}
+
+		GdipFree (old_scan0);
 	}
 
 	bitmap->data.Scan0 = rotated;
 	bitmap->data.Reserved |= GBD_OWN_SCAN0;	
+
+	if (image->surface != NULL) {
+		cairo_surface_destroy (image->surface);
+		image->surface = NULL;
+	}
+
+	return Ok;
 }
 
-void
-gdip_rotate_180 (GpImage *image)
+static GpStatus
+gdip_rotate_flip_packed_indexed (GpImage *image, PixelFormat pixel_format, int angle, BOOL flip_x)
 {
-	BYTE *src, *trg, *rotated;
-	int stride, height, line, x;	
-	GpBitmap *bitmap = (GpBitmap *) image;
-	int components = gdip_get_pixel_format_components (bitmap->data.PixelFormat);		         
-	 
-	bitmap = (GpBitmap *) image;	
-        stride = bitmap->data.Stride;
-	height = bitmap->data.Height;
-	rotated = malloc (stride * height);	
-	src = (BYTE *) bitmap->data.Scan0;
-	
-	for (line = 0; line < height; line++, src += stride) {		
-		for (x = 0; x < bitmap->data.Width; x++) {		
-			trg = rotated;	
-			trg += (bitmap->data.Height - 1  - line) * stride;
-			trg += (bitmap->data.Width - 1  - x) * components;		
-			copy_pixel (src + (x*components), trg, components);			
-		}	
-	}	
-	
-	memcpy (bitmap->data.Scan0, rotated, stride * height);	
-	free (rotated);
-}
+	GpBitmap *bitmap = (GpBitmap *)image;
+	BitmapData *data = &bitmap->data;
+	BYTE *rotated;
+	StreamingState scan[8];
+	BOOL scan_valid[8];
+	int x, y, i;
 
-void
-gdip_rotate_270 (GpImage *image)
-{
-	BYTE *src, *trg, *rotated;
-	int line, x, stride_trg, height_trg, width_trg;
-	GpBitmap *bitmap = (GpBitmap *) image;
-	int components = gdip_get_pixel_format_components (bitmap->data.PixelFormat);
+	int bits_per_pixel = gdip_get_pixel_format_depth (pixel_format);
+	int pixels_per_byte = 8 / bits_per_pixel;
 
-	height_trg = bitmap->data.Width;
-	width_trg = bitmap->data.Height;
-	stride_trg = components * gdip_get_pixel_format_depth (bitmap->data.PixelFormat);
-	stride_trg = (stride_trg * width_trg) / 8;
-	stride_trg = (stride_trg + 3) & ~3;		
+	int source_width = data->Width;
+	int source_height = data->Height;
 
-	rotated = malloc (stride_trg * height_trg);
-	
-	src = (BYTE *) bitmap->data.Scan0;
-	
-	for (line = 0; line < bitmap->data.Height; line++, src += bitmap->data.Stride) {		
-		for (x = 0; x < bitmap->data.Width; x++) {
+	/* Swap the width & height if needed */
+	BOOL aspect_inversion = ((angle % 180) != 0);
 
-			if (x >= height_trg)
-				continue;			
+	int target_width = aspect_inversion ? source_height : source_width;
+	int target_height = aspect_inversion ? source_width : source_height;
 
-			if (line >= width_trg)
-				continue;
+	int target_scan_size = (target_width + pixels_per_byte - 1) / pixels_per_byte;
+	int target_stride = (target_scan_size + 3) & ~3;
 
-			trg = rotated;	
-			trg += (height_trg -1 -x) * stride_trg;
-			trg += line * components;	
-			copy_pixel (src + (x * components), trg, components);			
-		}	
-	}	
+	if ((angle == 180) && flip_x)
+		return gdip_flip_y(image);
 
-	bitmap->data.Stride = stride_trg;
-	bitmap->data.Height = image->height = height_trg;
-	bitmap->data.Width = image->width = width_trg;
+	rotated = GdipAlloc (target_height * target_stride);
+
+	if (rotated == NULL)
+		return OutOfMemory;
+
+	if (aspect_inversion == FALSE) {
+		int x_alignment = ((source_width - 1) % pixels_per_byte) + 1 - pixels_per_byte;
+		int x_step = pixels_per_byte;
+
+		StreamingState stream;
+
+		BOOL target_starts_at_bottom = (angle == 180);
+
+		int target_x_offset = target_scan_size - 1;
+		int target_y_offset = target_starts_at_bottom ? (target_height - 1) * target_stride : 0;
+		int target_y_offset_delta = target_starts_at_bottom ? -target_stride : +target_stride;
+
+		Status status = gdip_init_pixel_stream (&stream, data, 0, 0, data->Width, data->Height);
+
+		if (status != Ok) {
+			GdipFree (rotated);
+			return status;
+		}
+
+		for (y = 0; y < source_height; y++, target_y_offset += target_y_offset_delta) {
+			BYTE *target = rotated + target_x_offset + target_y_offset;
+
+			for (x = x_alignment; x < source_width; x += x_step) {
+				int byte = 0;
+				for (i = 0; i < pixels_per_byte; i++) {
+					int index = x + i;
+
+					if ((index >= 0) && (index < source_width))
+						byte |= (gdip_pixel_stream_get_next (&stream) << 8);
+					byte >>= bits_per_pixel;
+				}
+
+				*target = byte;
+				target--;
+			}
+		}
+	}
+	else {
+		int y_alignment = ((source_height - 1) % pixels_per_byte) + 1 - pixels_per_byte;
+		int y_step = pixels_per_byte;
+		int byte_column = 0;
+
+		int target_delta;
+
+		BOOL target_starts_at_bottom = (angle == 270);
+		BOOL target_starts_at_left = (target_starts_at_bottom ^ flip_x);
+
+		if (target_starts_at_left)
+			y_alignment = 0;
+
+		if (target_starts_at_bottom)
+			target_delta = -target_stride;
+		else
+			target_delta = +target_stride;
+
+		for (y = y_alignment; y < source_height; y += y_step) {
+			BYTE *target;
+
+			for (i=0; i < pixels_per_byte; i++) {
+				int scan_index = y + i;
+
+				scan_valid[i] = ((scan_index >= 0) && (scan_index < source_height));
+
+				if (scan_valid[i]) {
+					Status status = gdip_init_pixel_stream (&scan[i], data, 0, scan_index, source_width, 1);
+
+					if (status != Ok) {
+						GdipFree (rotated);
+						return status;
+					}
+				}
+			}
+
+			target = rotated;
+
+			if (target_starts_at_left)
+				target += byte_column;
+			else
+				target += target_scan_size - byte_column - 1;
+
+			if (target_starts_at_bottom)
+				target += (target_height - 1) * target_stride;
+
+			for (x = 0; x < source_width; x++) {
+				int byte = 0;
+
+				if (target_starts_at_left)
+					for (i=0; i < pixels_per_byte; i++)
+					{
+						byte <<= bits_per_pixel;
+						if (scan_valid[i])
+							byte |= gdip_pixel_stream_get_next (&scan[i]);
+					}
+				else
+					for (i = pixels_per_byte - 1; i >= 0; i--)
+					{
+						byte <<= bits_per_pixel;
+						if (scan_valid[i])
+							byte |= gdip_pixel_stream_get_next (&scan[i]);
+					}
+
+				*target = byte;
+				target += target_delta;
+			}
+
+			byte_column++;
+		}
+	}
+
+	bitmap->data.Stride = target_stride;
+	bitmap->data.Height = image->height = target_height;
+	bitmap->data.Width = image->width = target_width;
 
 	if ((bitmap->data.Reserved & GBD_OWN_SCAN0) != 0) {
-		int k,j; 
+		int k, j; 
+
+		void *old_scan0 = bitmap->data.Scan0;
 
-		/* Do the free first since the frame point can be for some codes a reference to bitmap->data*/
-		free (bitmap->data.Scan0);
 		for (j = 0; j < image->frameDimensionCount; j++) {
 			for (k = 0; k < image->frameDimensionList [j].count; k++) {
-				if (image->frameDimensionList[j].frames[k].Scan0 == bitmap->data.Scan0)  {
+				if (image->frameDimensionList[j].frames[k].Scan0 == old_scan0)  {
 					image->frameDimensionList[j].frames[k].Scan0 = rotated;
-					image->frameDimensionList[j].frames[k].Stride = stride_trg;
-					image->frameDimensionList[j].frames[k].Height = image->height = height_trg;
-					image->frameDimensionList[j].frames[k].Width = image->width = width_trg;
-				}				
+					image->frameDimensionList[j].frames[k].Stride = target_stride;
+					image->frameDimensionList[j].frames[k].Height = image->height = target_height;
+					image->frameDimensionList[j].frames[k].Width = image->width = target_width;
+				}
 			}
-		}		
+		}
+
+		GdipFree (old_scan0);
 	}
 
 	bitmap->data.Scan0 = rotated;
-	bitmap->data.Reserved |= GBD_OWN_SCAN0;
-}
+	bitmap->data.Reserved |= GBD_OWN_SCAN0;	
 
-void
-gdip_FlipX (GpImage *image)
-{
-	BYTE *src, *trg, *rotated;
-	int stride, height, line, x;	
-	GpBitmap *bitmap = (GpBitmap *) image;
-	int components = gdip_get_pixel_format_components (bitmap->data.PixelFormat);		         
-	 
-	bitmap = (GpBitmap *) image;	
-        stride = bitmap->data.Stride;
-	height = bitmap->data.Height;
-	rotated = malloc (stride * height);
-	
-	src = (BYTE *) bitmap->data.Scan0;
-	
-	for (line = 0; line < height; line++, src += stride) {		
-		for (x = 0; x < bitmap->data.Width; x++) {
-		
-			trg = rotated;	
-			trg += (bitmap->data.Width-1-x)  * components;
-			trg += line * stride;	
-			copy_pixel (src + (x*components), trg, components);			
-		}	
-	}	
-	
-	memcpy (bitmap->data.Scan0, rotated, stride * height);	
-	free (rotated);
+	/* It shouldn't be possible for an indexed image to have one,
+	 * but if it does, it needs to be killed.
+	 */
+	if (image->surface != NULL) {
+		cairo_surface_destroy (image->surface);
+		image->surface = NULL;
+	}
+
+	return Ok;
 }
 
 GpStatus 
 GdipImageRotateFlip (GpImage *image, RotateFlipType type)
 {
+	int angle;
+	BOOL flip_x;
+	PixelFormat pixel_format;
+
 	if (!image)
 		return InvalidParameter;
 
+	pixel_format = ((GpBitmap *)image)->data.PixelFormat;
+
+
+	angle = flip_x = 0;
+
 	switch (type) {
-                case RotateNoneFlipNone:
+                case RotateNoneFlipNone: /* equivalent to Rotate180FlipXY */
+			return Ok;
+		case Rotate90FlipNone: /* equivalent to Rotate270FlipXY */
+			angle = 90;
 			break;
-		case Rotate90FlipNone:
-			gdip_rotate_90 (image);
+		case RotateNoneFlipXY: /* equivalent to Rotate180FlipNone */
+			angle = 180;
 			break;
-		case Rotate180FlipNone:
-			gdip_rotate_180 (image);
+		case Rotate90FlipXY: /* equivalent to Rotate270FlipNone */
+			angle = 270;
 			break;
-		case Rotate270FlipNone:	
-			gdip_rotate_270 (image);
+		case RotateNoneFlipX: /* equivalent to Rotate180FlipY */
+			flip_x = TRUE;
 			break;
-		case RotateNoneFlipX:
-			gdip_FlipX (image);
+		case Rotate90FlipX: /* equivalent to Rotate270FlipY */
+			angle = 90;
+			flip_x = TRUE;
 			break;
-		case Rotate90FlipX:
-			gdip_rotate_90 (image);
-			gdip_FlipX (image);
-			break;
-		case Rotate180FlipX:
-			gdip_rotate_180_FlipX (image);
-			break;
-		case Rotate270FlipX:
-			gdip_rotate_270 (image);
-			gdip_FlipX (image);
+		case RotateNoneFlipY: /* equivalent to Rotate180FlipX */
+			return gdip_flip_y (image);
+		case Rotate90FlipY: /* equivalent to Rotate270FlipX */
+			angle = 270;
+			flip_x = TRUE;
 			break;			
 		default:
 			return NotImplemented;
 	}	
 	
-	return Ok;
+	if (gdip_is_an_indexed_pixelformat (pixel_format)
+         && (gdip_get_pixel_format_depth (pixel_format) < 8))
+		return gdip_rotate_flip_packed_indexed (image, pixel_format, angle, flip_x);
+	else
+		return gdip_rotate_orthogonal_flip_x (image, angle, flip_x);
 }
 
 GpStatus 
 GdipGetImagePalette (GpImage *image, ColorPalette *palette, int size)
 {
-	return NotImplemented;
+	int palette_entries;
+	int bytes_needed;
+
+	if ((image == NULL) || (palette == NULL))
+		return InvalidParameter;
+
+	if (image->palette == NULL)
+		return InvalidParameter;
+
+	palette_entries = image->palette->Count;
+
+	if ((image->type == imageBitmap) && (((GpBitmap *)image)->data.PixelFormat == Format4bppIndexed))
+		palette_entries = 16;
+
+	bytes_needed = palette_entries * sizeof(ARGB) + sizeof(ColorPalette) - sizeof(ARGB);
+
+	if (bytes_needed > size)
+		return InvalidParameter;
+
+	memcpy(palette, image->palette, bytes_needed);
+	return Ok;
 }
 
 GpStatus 
 GdipSetImagePalette (GpImage *image, GDIPCONST ColorPalette *palette)
 {
+	int entries_to_copy;
+
+	if ((image == NULL) || (palette == NULL))
+		return InvalidParameter;
+
+	if (image->palette == NULL)
+		return InvalidParameter;
+
+	entries_to_copy = image->palette->Count;
+	if (entries_to_copy > palette->Count)
+		entries_to_copy = palette->Count;
+
+	memcpy(image->palette->Entries, palette->Entries, entries_to_copy * sizeof(ARGB));
 	return Ok;
 }
 
 GpStatus 
 GdipGetImagePaletteSize (GpImage *image, int* size)
 {
-	return NotImplemented;
+        int palette_entries;
+
+        if ((image == NULL) || (size == NULL))
+                return InvalidParameter;
+
+        if (image->palette == NULL)
+                return InvalidParameter;
+
+        palette_entries = image->palette->Count;
+
+        if ((image->type == imageBitmap) && (((GpBitmap *)image)->data.PixelFormat == Format4bppIndexed))
+                palette_entries = 16;
+
+        *size = palette_entries * sizeof(ARGB) + sizeof(ColorPalette) - sizeof(ARGB);
+	return Ok;
 }
 
 GpStatus 
@@ -1130,14 +1442,14 @@ GdipSetPropertyItem(GpImage *image, GDIPCONST PropertyItem *item)
 void
 gdip_image_clone (GpImage* image, GpImage* clonedImage)
 {
-	int i = 0, j = 0, count = 0, dataCount = 0;
+	int i = 0, j = 0, dataCount = 0;
 	BitmapData *data, *clonedData;
 
 	clonedImage->surface = NULL;
 
 	if (image->frameDimensionCount) {
 		clonedImage->frameDimensionCount = image->frameDimensionCount;
-		clonedImage->frameDimensionList = malloc (sizeof (FrameInfo) * image->frameDimensionCount);
+		clonedImage->frameDimensionList = GdipAlloc (sizeof (FrameInfo) * image->frameDimensionCount);
 
 		for (i = 0; i < image->frameDimensionCount; i++) {
 			clonedImage->frameDimensionList[i].count = image->frameDimensionList[i].count;
@@ -1146,7 +1458,7 @@ gdip_image_clone (GpImage* image, GpImage* clonedImage)
 
 			dataCount = image->frameDimensionList[i].count;
 			data = image->frameDimensionList[i].frames;
-			clonedImage->frameDimensionList[i].frames = malloc (sizeof (BitmapData) * dataCount);
+			clonedImage->frameDimensionList[i].frames = GdipAlloc (sizeof (BitmapData) * dataCount);
 			clonedData = clonedImage->frameDimensionList[i].frames;
 			/* Copy all BitmapData */
 			memcpy (clonedImage->frameDimensionList[i].frames, 
@@ -1154,11 +1466,11 @@ gdip_image_clone (GpImage* image, GpImage* clonedImage)
 
 			for (j = 0; j < dataCount; j++) {
 				if (data[j].Scan0) {
-					clonedData[j].Scan0 = malloc (data[j].Stride * data[j].Height);
+					clonedData[j].Scan0 = GdipAlloc (data[j].Stride * data[j].Height);
 					memcpy (clonedData[j].Scan0, data[j].Scan0, data[j].Stride * data[j].Height);				
 				}
 				if ((data[j].ByteCount) > 0 && (data[j].Bytes != NULL)) {				
-					clonedData[j].Bytes = malloc (data[j].ByteCount);
+					clonedData[j].Bytes = GdipAlloc (data[j].ByteCount);
 					memcpy (clonedData[j].Bytes, data[j].Bytes, data[j].ByteCount);
 				}
 			}
@@ -1192,7 +1504,7 @@ GdipCloneImage(GpImage *image, GpImage **cloneImage)
 ImageFormat 
 get_image_format (char *sig_read, size_t size_read)
 {
-	int index, sig_len, sig_num, inner_index;
+	int index;
 	char png[] = { 0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, '\0'};
 	char *signature[]  = { "BM", "MM", "II", "GIF", png, "\xff\xd8", "\xff\xd8\xff\xe1", "", "", ""};
 
@@ -1311,11 +1623,11 @@ gdip_get_pixel_format_components(PixelFormat pixfmt)
 
 GpStatus
 GdipLoadImageFromDelegate_linux (GetHeaderDelegate getHeaderFunc,
-				 GetBytesDelegate getBytesFunc,
-				 PutBytesDelegate putBytesFunc,
+								 GetBytesDelegate getBytesFunc,
+								 PutBytesDelegate putBytesFunc,
                                  SeekDelegate seekFunc,
-				 CloseDelegate closeFunc,
-				 SizeDelegate sizeFunc,
+								 CloseDelegate closeFunc,
+								 SizeDelegate sizeFunc,
                                  GpImage **image)
 {
 	GpImage *result = 0;
@@ -1356,6 +1668,7 @@ GdipLoadImageFromDelegate_linux (GetHeaderDelegate getHeaderFunc,
 				result->format = GIF;            
 			break;
 		default:
+			printf ("type: %d Not implemented\n", format);
 			status = NotImplemented;
 			break;
 	}
@@ -1445,15 +1758,6 @@ gdip_image_format_for_format_guid (GDIPCONST GUID *formatGUID)
 	return INVALID;
 }
 
-static char *
-guid_to_string_hack (GDIPCONST GUID *guid)
-{
-	static char buf[1024];
-	snprintf (buf, 1024, "%08lx-%04x-%04x-%02x-%02x-%02x-%02x-%02x-%02x-%02x-%02x",
-		  guid->Data1, guid->Data2, guid->Data3,
-		  guid->Data4[0], guid->Data4[1], guid->Data4[2], guid->Data4[3], guid->Data4[4], guid->Data4[5], guid->Data4[6], guid->Data4[7]);
-	return buf;
-}
 
 #define CODECS_SUPPORTED 5
 static BYTE *g_codeclist;
@@ -1463,7 +1767,7 @@ void initCodecList (void)
 {
 	BYTE *pos;
 	
-	g_codeclist = pos = malloc (sizeof (ImageCodecInfo) * CODECS_SUPPORTED);
+	g_codeclist = pos = GdipAlloc (sizeof (ImageCodecInfo) * CODECS_SUPPORTED);
 	
 	/* BMP codec - built-in */
 	memcpy (pos, gdip_getcodecinfo_bmp(), sizeof (ImageCodecInfo));
@@ -1502,7 +1806,7 @@ void initCodecList (void)
 void releaseCodecList (void)
 {
 	if (g_codeclist)
-		free (g_codeclist);
+		GdipFree (g_codeclist);
 }
 
 GpStatus
@@ -1609,7 +1913,7 @@ GdipGetImageThumbnail(GpImage *image, UINT thumbWidth, UINT thumbHeight, GpImage
 GpStatus
 GdipLoadImageFromFileICM(GDIPCONST WCHAR* filename, GpImage **image)
 {
-	return (GdipLoadImageFromFileICM(filename, image));
+	return (GdipLoadImageFromFile (filename, image));
 }
 
 const EncoderParameter *
