@@ -116,7 +116,7 @@ typedef struct gdip_jpeg_error_mgr *gdip_jpeg_error_mgr_ptr;
 static void
 _gdip_jpeg_error_exit (j_common_ptr cinfo)
 {
-    gdip_jpeg_error_mgr_ptr err = (gdip_jpeg_error_mgr_ptr) cinfo->err;
+    /*gdip_jpeg_error_mgr_ptr err = (gdip_jpeg_error_mgr_ptr) cinfo->err;*/
     char buffer[JMSG_LENGTH_MAX];
 
     (* cinfo->err->format_message) (cinfo, buffer);
@@ -295,7 +295,7 @@ gdip_load_jpeg_image_internal (struct jpeg_source_mgr *src,
         jpeg_destroy_decompress (&cinfo);
 
         if (destbuf != NULL)
-            g_free (destbuf);
+            GdipFree (destbuf);
 
         gdip_bitmap_dispose (img);
         *image = NULL;
@@ -451,7 +451,6 @@ gdip_load_jpeg_image_internal (struct jpeg_source_mgr *src,
 	    for (i = 0; i < nlines; i++) {
 		int j;
 		guchar *inptr, *outptr;
-		JOCTET r, g, b;
 
 		inptr = lines[i] + (img->image.width) * 3;
 		outptr = lines[i] + stride;
@@ -473,7 +472,7 @@ gdip_load_jpeg_image_internal (struct jpeg_source_mgr *src,
     img->data.Scan0 = destbuf;
     img->data.Reserved = GBD_OWN_SCAN0;
 
-    img->image.surface = cairo_surface_create_for_image ((char *)destbuf, img->cairo_format,
+    img->image.surface = cairo_image_surface_create_for_data ((unsigned char *)destbuf, img->cairo_format,
                                                    img->image.width, img->image.height,
                                                    stride);
 
@@ -567,7 +566,10 @@ gdip_save_jpeg_image_internal (FILE *fp,
         case Format32bppPArgb:
         case Format32bppRgb:
         case Format24bppRgb:
-        case Format8bppIndexed: /* assume this is grayscale */
+	    break;
+        case Format8bppIndexed: /* check that this is grayscale */
+	    if ((image->palette->Flags & PaletteFlagsGrayScale) == 0)
+		return InvalidParameter;
             break;
         default:
             return InvalidParameter;
