@@ -45,7 +45,7 @@ gdip_linear_gradient_init (GpLineGradient *linear)
 {
 	gdip_brush_init (&linear->base, &vtable);
 	linear->wrapMode = WrapModeTile;
-	linear->matrix = cairo_matrix_create ();
+	GdipCreateMatrix(&linear->matrix);
 	linear->rectangle = NULL;
 	linear->gammaCorrection = FALSE;
 	linear->angle = 0.0;
@@ -173,10 +173,6 @@ gdip_linear_gradient_destroy (GpBrush *brush)
 
 	linear = (GpLineGradient *) brush;
 
-	if (linear->matrix)
-		cairo_matrix_destroy (linear->matrix);
-	linear->matrix = NULL;
-
 	if (linear->rectangle)
 		GdipFree (linear->rectangle);
 	linear->rectangle = NULL;
@@ -228,7 +224,7 @@ add_color_stops_from_blend (cairo_pattern_t *pattern, Blend *blend, ARGB *colors
 		factor = blend->factors [index];
 		offset = blend->positions [index];
 
-		cairo_pattern_add_color_stop (pattern, offset, 
+		cairo_pattern_add_color_stop_rgba (pattern, offset, 
 					      ((sr * (1 - factor)) + (er * factor)) / 255,
 					      ((sg * (1 - factor)) + (eg * factor)) / 255,
 					      ((sb * (1 - factor)) + (eb * factor)) / 255,
@@ -256,7 +252,7 @@ add_color_stops_from_interpolation_colors (cairo_pattern_t *pattern, Interpolati
 		b = color & 0xFF;
 		offset = presetColors->positions [index];
 
-		cairo_pattern_add_color_stop (pattern, offset, r / 255, g / 255, b / 255, a / 255);
+		cairo_pattern_add_color_stop_rgba (pattern, offset, r / 255, g / 255, b / 255, a / 255);
 	}
 }
 
@@ -271,7 +267,7 @@ add_color_stops (cairo_pattern_t *pattern, ARGB *colors)
 	b = colors[0] & 0xFF;
 
 	/* set start color */
-	cairo_pattern_add_color_stop (pattern, 0.0, r / 255, g / 255, b / 255, a / 255);
+	cairo_pattern_add_color_stop_rgba (pattern, 0.0, r / 255, g / 255, b / 255, a / 255);
 
 	a = (colors[1] >> 24) & 0xFF;
 	r = (colors[1] >> 16) & 0xFF;
@@ -279,29 +275,25 @@ add_color_stops (cairo_pattern_t *pattern, ARGB *colors)
 	b = colors[1] & 0xFF;
 
 	/* set end color */
-	cairo_pattern_add_color_stop (pattern, 1.0, r / 255, g / 255, b / 255, a / 255);
+	cairo_pattern_add_color_stop_rgba (pattern, 1.0, r / 255, g / 255, b / 255, a / 255);
 }
 
 GpStatus
 create_tile_linear (cairo_t *ct, GpLineGradient *linear)
 {
+	cairo_t	*ct2;
 	cairo_surface_t *gradient;
 	cairo_pattern_t *pat;
-	GpMatrix *tempMatrix;
-	GpMatrix *currMatrix;
+	GpMatrix *tempMatrix = NULL;
 	GpRectF *rect = linear->rectangle;
 	g_return_val_if_fail (rect != NULL, InvalidParameter);
 
-	tempMatrix = cairo_matrix_create ();
-	g_return_val_if_fail (tempMatrix != NULL, OutOfMemory);
-
-	/* gradient surface to be created. We are using CAIRO_FORMAT_ARGB32, */
+	/* gradient surface to be created. We are using CAIRO_CONTENT_COLOR_ALPHA, */
 	/* because internally this is the format we use everywhere. */
-	gradient = cairo_surface_create_similar (cairo_current_target_surface (ct), CAIRO_FORMAT_ARGB32,
+	gradient = cairo_surface_create_similar (cairo_get_target (ct), CAIRO_CONTENT_COLOR_ALPHA,
 						 (int) (2 * rect->Width), (int) (2 * rect->Height));
 
 	if (gradient == NULL) {
-		cairo_matrix_destroy (tempMatrix);
 		return OutOfMemory;
 	}
 
@@ -309,33 +301,21 @@ create_tile_linear (cairo_t *ct, GpLineGradient *linear)
 					   linear->points [1].X, linear->points [1].Y);
 
 	if (pat == NULL) {
-		cairo_matrix_destroy (tempMatrix);
 		cairo_surface_destroy (gradient);
 		return OutOfMemory;
 	}
 
-	cairo_save (ct);
-	{
-		cairo_set_target_surface (ct, gradient);
-		cairo_default_matrix (ct);
+	ct2 = cairo_create(gradient);
 
-		if (linear->blend->count > 1)
-			add_color_stops_from_blend (pat, linear->blend, linear->lineColors);
-		else if (linear->presetColors->count > 1)
-			add_color_stops_from_interpolation_colors (pat, linear->presetColors);
-		else
-			add_color_stops (pat, linear->lineColors);
+	if (linear->blend->count > 1)
+		add_color_stops_from_blend (pat, linear->blend, linear->lineColors);
+	else if (linear->presetColors->count > 1)
+		add_color_stops_from_interpolation_colors (pat, linear->presetColors);
+	else
+		add_color_stops (pat, linear->lineColors);
 
-		cairo_pattern_set_extend (pat, CAIRO_EXTEND_REPEAT);
-		cairo_set_pattern (ct, pat);
-		cairo_rectangle (ct, 0, 0, 2 * rect->Width, 2 * rect->Height);
-		cairo_fill (ct);
+	GdipCreateMatrix (&tempMatrix);
 
-		cairo_pattern_destroy (pat);
-	}
-	cairo_restore (ct);
-
-	cairo_matrix_set_identity (tempMatrix);
 	/* rotate the pattern if angle is non-zero */
 	if (linear->angle != 0) {
 		/* Absolute rotation */
@@ -349,17 +329,20 @@ create_tile_linear (cairo_t *ct, GpLineGradient *linear)
 		cairo_matrix_multiply (tempMatrix, tempMatrix, linear->matrix);
 	}
 
+	/* pattern is transformed wrt to main graphics */
+	cairo_matrix_multiply (linear->matrix, linear->matrix, tempMatrix);
+
+	cairo_pattern_set_extend (pat, CAIRO_EXTEND_REPEAT);
+	cairo_set_source (ct2, pat);
+	cairo_rectangle (ct2, 0, 0, 2 * rect->Width, 2 * rect->Height);
+	cairo_fill (ct2);	
+
+	cairo_pattern_destroy (pat);
+	cairo_destroy (ct2);
+
 	linear->pattern = cairo_pattern_create_for_surface (gradient);
 	cairo_surface_destroy (gradient);
 
-	/* pattern is transformed wrt to main graphics */
-	currMatrix = cairo_matrix_create ();
-	cairo_current_matrix (ct, currMatrix);
-	cairo_matrix_multiply (tempMatrix, tempMatrix, currMatrix);
-	cairo_set_matrix (ct, tempMatrix);
-
-	cairo_matrix_destroy (tempMatrix);
-	cairo_matrix_destroy (currMatrix);
 
 	return Ok;
 }
@@ -369,21 +352,18 @@ create_tile_flipX_linear (cairo_t *ct, GpLineGradient *linear)
 {
 	cairo_surface_t *gradient;
 	cairo_pattern_t *pat;
-	GpMatrix *tempMatrix;
-	GpMatrix *currMatrix;
+	GpMatrix *tempMatrix = NULL;
+	GpMatrix *currMatrix = NULL;
 	GpRectF *rect = linear->rectangle;
 	g_return_val_if_fail (rect != NULL, InvalidParameter);
 
-	tempMatrix = cairo_matrix_create ();
-	g_return_val_if_fail (tempMatrix != NULL, OutOfMemory);
-
-	/* gradient surface to be created. We are using CAIRO_FORMAT_ARGB32, */
+	/* gradient surface to be created. We are using CAIRO_CONTENT_COLOR_ALPHA, */
 	/* because internally this is the format we use everywhere. */
-	gradient = cairo_surface_create_similar (cairo_current_target_surface (ct), CAIRO_FORMAT_ARGB32,
+	gradient = cairo_surface_create_similar (cairo_get_target (ct), CAIRO_CONTENT_COLOR_ALPHA,
 						 (int) (2 * rect->Width), (int) (2 * rect->Height));
 
 	if (gradient == NULL) {
-		cairo_matrix_destroy (tempMatrix);
+		
 		return OutOfMemory;
 	}
 
@@ -392,13 +372,14 @@ create_tile_flipX_linear (cairo_t *ct, GpLineGradient *linear)
 
 	if (pat == NULL) {
 		cairo_surface_destroy (gradient);
-		cairo_matrix_destroy (tempMatrix);
+		
 		return OutOfMemory;
 	}
-
+	
+	GdipCreateMatrix (&tempMatrix);
 	cairo_save (ct);
 	{
-		cairo_set_target_surface (ct, gradient);
+		ct = cairo_create (gradient);
 
 		if (linear->blend->count > 1)
 			add_color_stops_from_blend (pat, linear->blend, linear->lineColors);
@@ -407,9 +388,9 @@ create_tile_flipX_linear (cairo_t *ct, GpLineGradient *linear)
 		else
 			add_color_stops (pat, linear->lineColors);
 
-		cairo_default_matrix (ct);
+		cairo_identity_matrix (ct);
 		cairo_pattern_set_extend (pat, CAIRO_EXTEND_REPEAT);
-		cairo_set_pattern (ct, pat);
+		cairo_set_source (ct, pat);
 
 		/* left part */
 		cairo_rectangle (ct, 0, 0, rect->Width, 2 * rect->Height);
@@ -427,7 +408,7 @@ create_tile_flipX_linear (cairo_t *ct, GpLineGradient *linear)
 	}
 	cairo_restore (ct);
 
-	cairo_matrix_set_identity (tempMatrix);
+	cairo_matrix_init_identity (tempMatrix);
 	/* rotate the pattern if angle is non-zero */
 	if (linear->angle != 0) {
 		/* Absolute rotation */
@@ -445,13 +426,13 @@ create_tile_flipX_linear (cairo_t *ct, GpLineGradient *linear)
 	cairo_surface_destroy (gradient);
 
 	/* pattern is transformed wrt to main graphics */
-	currMatrix = cairo_matrix_create ();
-	cairo_current_matrix (ct, currMatrix);
+	GdipCreateMatrix (&currMatrix);
+	cairo_get_matrix (ct, currMatrix);
 	cairo_matrix_multiply (tempMatrix, tempMatrix, currMatrix);
 	cairo_set_matrix (ct, tempMatrix);
-
-	cairo_matrix_destroy (tempMatrix);
-	cairo_matrix_destroy (currMatrix);
+	GdipDeleteMatrix (currMatrix);
+	
+	
 
 	return Ok;
 }
@@ -461,21 +442,18 @@ create_tile_flipY_linear (cairo_t *ct, GpLineGradient *linear)
 {
 	cairo_surface_t *gradient;
 	cairo_pattern_t *pat;
-	GpMatrix *tempMatrix;
-	GpMatrix *currMatrix;
+	GpMatrix *tempMatrix = NULL;
+	GpMatrix *currMatrix = NULL;
 	GpRectF *rect = linear->rectangle;
 	g_return_val_if_fail (rect != NULL, InvalidParameter);
 
-	tempMatrix = cairo_matrix_create ();
-	g_return_val_if_fail (tempMatrix != NULL, OutOfMemory);
-
-	/* gradient surface to be created. We are using CAIRO_FORMAT_ARGB32, */
+	/* gradient surface to be created. We are using CAIRO_CONTENT_COLOR_ALPHA, */
 	/* because internally this is the format we use everywhere. */
-	gradient = cairo_surface_create_similar (cairo_current_target_surface (ct), CAIRO_FORMAT_ARGB32,
+	gradient = cairo_surface_create_similar (cairo_get_target (ct), CAIRO_CONTENT_COLOR_ALPHA,
 						 (int) (2 * rect->Width), (int) (2 * rect->Height));
 
 	if (gradient == NULL) {
-		cairo_matrix_destroy (tempMatrix);
+		
 		return OutOfMemory;
 	}
 
@@ -484,13 +462,14 @@ create_tile_flipY_linear (cairo_t *ct, GpLineGradient *linear)
 
 	if (pat == NULL) {
 		cairo_surface_destroy (gradient);
-		cairo_matrix_destroy (tempMatrix);
+		
 		return OutOfMemory;
 	}
 
+	GdipCreateMatrix (&tempMatrix);	
 	cairo_save (ct);
 	{
-		cairo_set_target_surface (ct, gradient);
+		ct = cairo_create (gradient);
 
 		if (linear->blend->count > 1)
 			add_color_stops_from_blend (pat, linear->blend, linear->lineColors);
@@ -499,9 +478,9 @@ create_tile_flipY_linear (cairo_t *ct, GpLineGradient *linear)
 		else
 			add_color_stops (pat, linear->lineColors);
 
-		cairo_default_matrix (ct);
+		cairo_identity_matrix (ct);
 		cairo_pattern_set_extend (pat, CAIRO_EXTEND_REPEAT);
-		cairo_set_pattern (ct, pat);
+		cairo_set_source (ct, pat);
 
 		/* upper part */
 		cairo_rectangle (ct, 0, 0, 2 * rect->Width, rect->Height);
@@ -519,7 +498,7 @@ create_tile_flipY_linear (cairo_t *ct, GpLineGradient *linear)
 	}
 	cairo_restore (ct);
 
-	cairo_matrix_set_identity (tempMatrix);
+	cairo_matrix_init_identity (tempMatrix);
 	/* rotate the pattern if angle is non-zero */
 	if (linear->angle != 0) {
 		/* Absolute rotation */
@@ -537,13 +516,12 @@ create_tile_flipY_linear (cairo_t *ct, GpLineGradient *linear)
 	cairo_surface_destroy (gradient);
 
 	/* pattern is transformed wrt to main graphics */
-	currMatrix = cairo_matrix_create ();
-	cairo_current_matrix (ct, currMatrix);
+	GdipCreateMatrix (&currMatrix);
+	cairo_get_matrix (ct, currMatrix);
 	cairo_matrix_multiply (tempMatrix, tempMatrix, currMatrix);
 	cairo_set_matrix (ct, tempMatrix);
-
-	cairo_matrix_destroy (tempMatrix);
-	cairo_matrix_destroy (currMatrix);
+	GdipDeleteMatrix (currMatrix);	
+	
 
 	return Ok;
 }
@@ -553,22 +531,19 @@ create_tile_flipXY_linear (cairo_t *ct, GpLineGradient *linear)
 {
 	cairo_surface_t *gradient;
 	cairo_pattern_t *pat;
-	GpMatrix *tempMatrix;
-	GpMatrix *currMatrix;
+	GpMatrix *tempMatrix = NULL;
+	GpMatrix *currMatrix = NULL;
 
 	GpRectF *rect = linear->rectangle;
 	g_return_val_if_fail (rect != NULL, InvalidParameter);
 
-	tempMatrix = cairo_matrix_create ();
-	g_return_val_if_fail (tempMatrix != NULL, OutOfMemory);
-
-	/* gradient surface to be created. We are using CAIRO_FORMAT_ARGB32, */
+	/* gradient surface to be created. We are using CAIRO_CONTENT_COLOR_ALPHA, */
 	/* because internally this is the format we use everywhere. */
-	gradient = cairo_surface_create_similar (cairo_current_target_surface (ct), CAIRO_FORMAT_ARGB32,
+	gradient = cairo_surface_create_similar (cairo_get_target (ct), CAIRO_CONTENT_COLOR_ALPHA,
 						 (int) (2 * rect->Width), (int) (2 * rect->Height));
-
+	
 	if (gradient == NULL) {
-		cairo_matrix_destroy (tempMatrix);
+		
 		return OutOfMemory;
 	}
 
@@ -577,13 +552,14 @@ create_tile_flipXY_linear (cairo_t *ct, GpLineGradient *linear)
 
 	if (pat == NULL) {
 		cairo_surface_destroy (gradient);
-		cairo_matrix_destroy (tempMatrix);
+		
 		return OutOfMemory;
 	}
 
+	GdipCreateMatrix (&tempMatrix);	
 	cairo_save (ct);
 	{
-		cairo_set_target_surface (ct, gradient);
+		ct = cairo_create (gradient);
 
 		if (linear->blend->count > 1)
 			add_color_stops_from_blend (pat, linear->blend, linear->lineColors);
@@ -594,7 +570,7 @@ create_tile_flipXY_linear (cairo_t *ct, GpLineGradient *linear)
 
 		cairo_identity_matrix (ct);
 		cairo_pattern_set_extend (pat, CAIRO_EXTEND_REPEAT);
-		cairo_set_pattern (ct, pat);
+		cairo_set_source (ct, pat);
 
 		/* Draw upper left part */
 		cairo_rectangle (ct, 0, 0, rect->Width, rect->Height);
@@ -616,7 +592,7 @@ create_tile_flipXY_linear (cairo_t *ct, GpLineGradient *linear)
 		cairo_translate (ct, rect->Width, -rect->Height);
 		cairo_rectangle (ct, 0, 0, rect->Width, rect->Height);
 		/* Reset the pattern matrix and do fresh transformation */
-		cairo_matrix_set_identity (tempMatrix);
+		cairo_matrix_init_identity (tempMatrix);
 		/* Not sure if this is a bug, but using rect->Width - 1
 		 * avoids the seam.
 		 */
@@ -642,7 +618,7 @@ create_tile_flipXY_linear (cairo_t *ct, GpLineGradient *linear)
 	}
 	cairo_restore (ct);
 
-	cairo_matrix_set_identity (tempMatrix);
+	cairo_matrix_init_identity (tempMatrix);
 	/* rotate the pattern if angle is non-zero */
 	if (linear->angle != 0) {
 		/* Absolute rotation */
@@ -660,13 +636,11 @@ create_tile_flipXY_linear (cairo_t *ct, GpLineGradient *linear)
 	cairo_surface_destroy (gradient);
 
 	/* pattern is transformed wrt to main graphics */
-	currMatrix = cairo_matrix_create ();
-	cairo_current_matrix (ct, currMatrix);
+	GdipCreateMatrix (&currMatrix);
+	cairo_get_matrix (ct, currMatrix);
 	cairo_matrix_multiply (tempMatrix, tempMatrix, currMatrix);
-	cairo_set_matrix (ct, tempMatrix);
-
-	cairo_matrix_destroy (tempMatrix);
-	cairo_matrix_destroy (currMatrix);
+	cairo_set_matrix (ct, tempMatrix);       
+	GdipDeleteMatrix (currMatrix);
 
 	return Ok;
 }
@@ -676,8 +650,7 @@ gdip_linear_gradient_setup (GpGraphics *graphics, GpBrush *brush)
 {
 	cairo_t *ct;
 	GpLineGradient *linear;
-	cairo_pattern_t *pattern;
-	GpMatrix *product;
+	GpMatrix *product = NULL;
 	GpStatus status = Ok;
 
 	g_return_val_if_fail (graphics != NULL, InvalidParameter);
@@ -688,9 +661,6 @@ gdip_linear_gradient_setup (GpGraphics *graphics, GpBrush *brush)
 
 	linear = (GpLineGradient *) brush;
 
-	product = cairo_matrix_create ();
-	g_return_val_if_fail (product != NULL, OutOfMemory);
-
 	/* We create the new pattern for brush, if the brush is changed
 	 * or if pattern has not been created yet.
 	 */
@@ -726,19 +696,18 @@ gdip_linear_gradient_setup (GpGraphics *graphics, GpBrush *brush)
 	if (status == Ok) {
 		if (linear->pattern != NULL) {
 			/* Use both the matrices */
+			GdipCreateMatrix (&product);
 			cairo_matrix_multiply (product, linear->matrix, graphics->copy_of_ctm);
 			cairo_matrix_invert (product);
 			cairo_pattern_set_matrix (linear->pattern, product);
 			cairo_pattern_set_extend (linear->pattern, CAIRO_EXTEND_REPEAT);
-			cairo_set_pattern (ct, linear->pattern);
+			cairo_set_source (ct, linear->pattern);
 
 			status = gdip_get_status (cairo_status (ct));
 		}
 		else
 			status = GenericError;
-	}
-
-	cairo_matrix_destroy (product);
+	}	
 
 	return status;
 }
@@ -746,8 +715,6 @@ gdip_linear_gradient_setup (GpGraphics *graphics, GpBrush *brush)
 void
 gdip_set_rect (GpRectF *rect, float x1, float y1, float x2, float y2)
 {
-	float x, y, width, height;
-
 	if (rect) {
 		if (x1 == x2) {
 			rect->Height = (y1 > y2) ? (y1 - y2) : (y2 - y1);
@@ -861,7 +828,6 @@ GdipCreateLineBrushFromRectI (GDIPCONST GpRect *rect, ARGB color1, ARGB color2,
 GpStatus
 GdipCreateLineBrushFromRect (GDIPCONST GpRectF *rect, ARGB color1, ARGB color2, LinearGradientMode mode, GpWrapMode wrapMode, GpLineGradient **lineGradient)
 {
-	GpLineGradient *linear;
 	float angle = 0.0;
 
 	switch (mode) {
@@ -1013,9 +979,6 @@ GdipSetLineBlend (GpLineGradient *brush, GDIPCONST float *blend, GDIPCONST float
 GpStatus
 GdipGetLineBlend (GpLineGradient *brush, float *blend, float *positions, int count)
 {
-	float *blendFactors;
-	float *blendPositions;
-
 	g_return_val_if_fail (brush != NULL, InvalidParameter);
 	g_return_val_if_fail (blend != NULL, InvalidParameter);
 	g_return_val_if_fail (positions != NULL, InvalidParameter);
@@ -1122,9 +1085,6 @@ GdipSetLinePresetBlend (GpLineGradient *brush, GDIPCONST ARGB *blend, GDIPCONST
 GpStatus
 GdipGetLinePresetBlend (GpLineGradient *brush, ARGB *blend, float *positions, int count)
 {
-	float *blendColors;
-	float *blendPositions;
-
 	g_return_val_if_fail (brush != NULL, InvalidParameter);
 	g_return_val_if_fail (blend != NULL, InvalidParameter);
 	g_return_val_if_fail (positions != NULL, InvalidParameter);
@@ -1202,7 +1162,7 @@ GdipGetLineTransform (GpLineGradient *brush, GpMatrix *matrix)
 	if (brush->presetColors->count >= 2)
 		return WrongState;
 
-	cairo_matrix_copy(matrix, brush->matrix);
+	gdip_cairo_matrix_copy(matrix, &brush->matrix);
 
 	return Ok;
 }
@@ -1213,9 +1173,7 @@ GdipSetLineTransform (GpLineGradient *brush, GDIPCONST GpMatrix *matrix)
 	g_return_val_if_fail (brush != NULL, InvalidParameter);
 	g_return_val_if_fail (matrix != NULL, InvalidParameter);
 
-	brush->matrix = cairo_matrix_create();
-	g_return_val_if_fail (brush->matrix != NULL, OutOfMemory);
-	cairo_matrix_copy(brush->matrix, matrix);
+	gdip_cairo_matrix_copy(&brush->matrix, matrix);
 	brush->base.changed = TRUE;
 	return Ok;
 }
@@ -1504,15 +1462,12 @@ GdipMultiplyLineTransform (GpLineGradient *brush, GpMatrix *matrix, GpMatrixOrde
 GpStatus
 GdipResetLineTransform (GpLineGradient *brush)
 {
-	GpStatus s;
-	cairo_status_t status;
 	g_return_val_if_fail (brush != NULL, InvalidParameter);
 
-	status = cairo_matrix_set_identity (brush->matrix);
-	s = gdip_get_status (status);
-	if (s == Ok)
-		brush->base.changed = TRUE;
-	return s;
+	cairo_matrix_init_identity (brush->matrix);
+       
+	brush->base.changed = TRUE;
+	return Ok;
 }
 
 GpStatus
