@@ -33,6 +33,7 @@ gdip_pen_init (GpPen *pen)
 {
         pen->color = 0;
         pen->brush = 0;
+	pen->own_brush = FALSE;
         pen->width = 1;
         pen->miter_limit = 10;
         pen->line_join = LineJoinMiter;
@@ -46,8 +47,10 @@ gdip_pen_init (GpPen *pen)
 	pen->compound_count = 0;
 	pen->compound_array = NULL;
 	pen->unit = UnitWorld;
-        pen->matrix = cairo_matrix_create ();
 	pen->changed = TRUE;
+	pen->matrix = NULL;
+	pen->matrix = g_new (GpMatrix, 1);
+	cairo_matrix_init_identity (pen->matrix);
 }
 
 GpPen*
@@ -86,13 +89,13 @@ convert_line_cap (GpLineCap cap)
 {
         switch (cap) {
         
+        case LineCapFlat:
         case LineCapSquare:
                 return CAIRO_LINE_CAP_SQUARE;
 
         case LineCapRound:
                 return CAIRO_LINE_CAP_ROUND;                
 
-        case LineCapFlat:
         case LineCapTriangle:
         case LineCapNoAnchor:
         case LineCapSquareAnchor:
@@ -106,12 +109,12 @@ convert_line_cap (GpLineCap cap)
 }
 
 static double *
-convert_dash_array (float *f, int count)
+convert_dash_array (float *f, float width, int count)
 {
-        double *retval = malloc (sizeof (double) * count);
+        double *retval = GdipAlloc (sizeof (double) * count);
         int i;
         for (i = 0; i < count; i++) {
-                retval[i] = (double) f[i];
+                retval[i] = (double) f[i] * width;
         }
 
         return retval;
@@ -121,11 +124,14 @@ GpStatus
 gdip_pen_setup (GpGraphics *graphics, GpPen *pen)
 {
 	GpStatus status;
-	cairo_matrix_t *product;
-
+	GpMatrix *product = NULL;
+	
 	g_return_val_if_fail (graphics != NULL, InvalidParameter);
 	g_return_val_if_fail (pen != NULL, InvalidParameter);
 
+	GdipCreateMatrix (&product);
+	cairo_matrix_init_identity (product);
+	
 	status = gdip_brush_setup (graphics, pen->brush);
 	if (status != Ok)
 		return status;
@@ -137,10 +143,8 @@ gdip_pen_setup (GpGraphics *graphics, GpPen *pen)
 	 * every time we perform stroke operations. Graphics matrix gets
 	 * reset to its own state after stroking.
 	 */
-	product = cairo_matrix_create ();
 	cairo_matrix_multiply (product, pen->matrix, graphics->copy_of_ctm);
 	cairo_set_matrix (graphics->ct, product);
-	cairo_matrix_destroy (product);
 
 	/* Don't need to setup, if pen is the same as the cached pen and
 	 * it is not changed. Just comparing pointers may not be sufficient
@@ -154,7 +158,7 @@ gdip_pen_setup (GpGraphics *graphics, GpPen *pen)
 	if (pen->width <= 0) { /* we draw a pixel wide line if width is <=0 */
 		double widthx = 1.0;
 		double widthy = 1.0;
-		cairo_inverse_transform_distance (graphics->ct, &widthx, &widthy);
+		cairo_device_to_user_distance (graphics->ct, &widthx, &widthy);
 		cairo_set_line_width (graphics->ct, widthx);
 	} else
 		cairo_set_line_width (graphics->ct, (double) pen->width);
@@ -166,9 +170,9 @@ gdip_pen_setup (GpGraphics *graphics, GpPen *pen)
         if (pen->dash_count > 0) {
                 double *dash_array;
 
-                dash_array = convert_dash_array (pen->dash_array, pen->dash_count);
+                dash_array = convert_dash_array (pen->dash_array, pen->width, pen->dash_count);
                 cairo_set_dash (graphics->ct, dash_array, pen->dash_count, pen->dash_offset);
-                free (dash_array);
+                GdipFree (dash_array);
         } else /* Clear the dashes, if set in previous calls */
 		cairo_set_dash (graphics->ct, NULL, 0, 0);
 
@@ -197,7 +201,7 @@ GdipCreatePen1 (int argb, float width, GpUnit unit, GpPen **pen)
 
 	s = GdipCreateSolidFill (argb, &solidBrush);
         (*pen)->brush = (GpBrush*) solidBrush;
-
+	(*pen)->own_brush = TRUE;
         return s;
 }
 
@@ -212,6 +216,7 @@ GdipCreatePen2 (GpBrush *brush, float width, GpUnit unit, GpPen **pen)
 	g_return_val_if_fail (pen != NULL, InvalidParameter);
 
         *pen = gdip_pen_new ();
+
 	g_return_val_if_fail (*pen != NULL, OutOfMemory);
 
         /* FIXME: do unit conversion when setting width */
@@ -257,7 +262,7 @@ GpStatus
 GdipClonePen (GpPen *pen, GpPen **clonepen)
 {
         GpPen *result;
-        GpMatrix *matrix;               /* copy of pen->matrix */
+        GpMatrix *matrix = NULL;        /* copy of pen->matrix */
         float *dashes;                  /* copy off pen->dash_array */
         float *compound_array = NULL;   /* copy off pen->compound_array */
 
@@ -267,6 +272,7 @@ GdipClonePen (GpPen *pen, GpPen **clonepen)
 	/* we make a copy of dash array only if it is owned by pen, i.e. it is not
 	 * our global array.
 	 */
+
 	if (pen->dash_count > 0 && pen->own_dash_array) {
 		dashes = (float *) GdipAlloc (pen->dash_count * sizeof (float));
 		g_return_val_if_fail (dashes != NULL, OutOfMemory);
@@ -284,6 +290,7 @@ GdipClonePen (GpPen *pen, GpPen **clonepen)
 		clone_dash_array (compound_array, pen->compound_array, pen->compound_count);
 	}
 
+	GdipCreateMatrix (&matrix);
         if (GdipCloneMatrix (pen->matrix, &matrix) != Ok) {
 		if (pen->dash_count > 0)
 			GdipFree (dashes);
@@ -304,8 +311,17 @@ GdipClonePen (GpPen *pen, GpPen **clonepen)
 		return OutOfMemory;
 	}
 
-        result->color = pen->color;
-	result->brush = pen->brush;
+	if (pen->own_brush) {
+		GpSolidFill* solidBrush;
+		GpSolidFill* oldbrush = (GpSolidFill*)pen->brush;
+		GdipCreateSolidFill (oldbrush->color, &solidBrush);
+		result->brush = (GpBrush*) solidBrush;
+	} else {
+		result->brush = pen->brush;
+	}
+
+        result->own_brush = pen->own_brush;
+	result->color = pen->color;
         result->width = pen->width;
         result->miter_limit = pen->miter_limit;
         result->line_join = pen->line_join;
@@ -319,7 +335,7 @@ GdipClonePen (GpPen *pen, GpPen **clonepen)
 	result->compound_count = pen->compound_count;
 	result->compound_array = compound_array;
 	result->unit = pen->unit;
-        result->matrix = matrix;
+        gdip_cairo_matrix_copy(result->matrix, matrix);
 	result->changed = pen->changed;
 
         *clonepen = result;
@@ -332,16 +348,15 @@ GdipDeletePen (GpPen *pen)
 {
 	g_return_val_if_fail (pen != NULL, InvalidParameter);
 
-        if (pen->matrix != NULL)
-                cairo_matrix_destroy (pen->matrix);
-	pen->matrix = NULL;
-
         if (pen->dash_count != 0 && pen->own_dash_array) {
                 GdipFree (pen->dash_array);
 		pen->dash_count = 0;
 		pen->dash_array = NULL;
 	}
 
+	if (pen->own_brush && pen->brush)
+		GdipFree (pen->brush);
+
         if (pen->compound_count != 0)
                 GdipFree (pen->compound_array);
 	pen->compound_array = NULL;
@@ -395,8 +410,12 @@ GdipSetPenBrushFill (GpPen *pen, GpBrush *brush)
 	else
 		pen->color = 0;
 
+	if (pen->own_brush && pen->brush)
+		GdipFree (pen->brush);
+
         pen->brush = brush;
 	pen->changed = TRUE;
+	pen->own_brush = FALSE;
 
         return Ok;
 }
@@ -548,8 +567,8 @@ GdipSetPenTransform (GpPen *pen, GDIPCONST GpMatrix *matrix)
 	g_return_val_if_fail (pen != NULL, InvalidParameter);
 	g_return_val_if_fail (matrix != NULL, InvalidParameter);
 
-	pen->matrix = cairo_matrix_create();
-	cairo_matrix_copy(pen->matrix, matrix);
+
+	gdip_cairo_matrix_copy(&pen->matrix, matrix);
 	pen->changed = TRUE;
         return Ok;
 }
@@ -560,24 +579,20 @@ GdipGetPenTransform (GpPen *pen, GpMatrix *matrix)
 	g_return_val_if_fail (pen != NULL, InvalidParameter);
 	g_return_val_if_fail (matrix != NULL, InvalidParameter);
 
-	cairo_matrix_copy(matrix, pen->matrix);
+	gdip_cairo_matrix_copy(matrix, &pen->matrix);
         return Ok;
 }
 
 GpStatus
 GdipResetPenTransform (GpPen *pen)
 {
-	cairo_status_t status;
-	GpStatus s;
 	g_return_val_if_fail (pen != NULL, InvalidParameter);
 
-	status = cairo_matrix_set_identity (pen->matrix);
-	s = gdip_get_status (status);
+	cairo_matrix_init_identity (pen->matrix);
 
-	if (s == Ok)
-		pen->changed = TRUE;
+	pen->changed = TRUE;
 
-	return s;
+	return Ok;
 }
 
 GpStatus
@@ -648,7 +663,7 @@ GdipGetPenDashStyle (GpPen *pen, GpDashStyle *dashStyle)
 }
 
 static float Custom [] = { 1.0 };
-static float Dot []  = { 1.0, 1.0 };
+static float Dot []  = { 1.0, 2.0 };	// FIXME - this is a workaround for cairo, should be 1.0, 1.0
 static float Dash []  = { 3.0, 1.0 };
 static float DashDot [] = { 3.0, 1.0, 1.0, 1.0 };
 static float DashDotDot [] = { 3.0, 1.0, 1.0, 1.0, 1.0, 1.0 };
