@@ -45,9 +45,9 @@ gdip_texture_init (GpTexture *texture)
 {
 	gdip_brush_init (&texture->base, &vtable);
 	texture->wrapMode = WrapModeTile;
-	texture->matrix = cairo_matrix_create ();
 	texture->rectangle = NULL;
 	texture->pattern = NULL;
+	GdipCreateMatrix (&texture->matrix);
 }
 
 GpTexture*
@@ -69,7 +69,7 @@ draw_tile_texture (cairo_t *ct, GpBitmap *bitmap, GpTexture *brush)
 {
 	cairo_surface_t *original;
 	cairo_surface_t *texture;
-	cairo_pattern_t *pat;
+	cairo_pattern_t *pat, *pattern;
 	GpRect *rect = brush->rectangle;
 	g_return_val_if_fail (rect != NULL, InvalidParameter);
 
@@ -83,7 +83,7 @@ draw_tile_texture (cairo_t *ct, GpBitmap *bitmap, GpTexture *brush)
 	cairo_pattern_set_extend (pat, CAIRO_EXTEND_REPEAT);
 
 	/* texture surface to be created */
-	texture = cairo_surface_create_similar (original, bitmap->cairo_format,
+	texture = cairo_surface_create_similar (original, from_cairoformat_to_content (bitmap->cairo_format),
 						2 * rect->Width, 2 * rect->Height);
 
 	g_return_val_if_fail (texture != NULL, OutOfMemory);
@@ -91,14 +91,15 @@ draw_tile_texture (cairo_t *ct, GpBitmap *bitmap, GpTexture *brush)
 	cairo_save (ct);
 	{
 		/* Draw the texture */
-		cairo_set_target_surface (ct, texture);
+		ct = cairo_create (texture);
 		cairo_identity_matrix (ct);
-		cairo_set_pattern (ct, pat);
+		cairo_set_source (ct, pat);
 		cairo_rectangle (ct, 0, 0, 2 * rect->Width, 2 * rect->Height);
 		cairo_fill (ct);
 	}
 	cairo_restore (ct);
-	cairo_surface_set_repeat (texture, 1);
+	pattern = cairo_pattern_create_for_surface (texture);
+	cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
 	brush->pattern = cairo_pattern_create_for_surface (texture);
 
 	cairo_pattern_destroy (pat);
@@ -112,8 +113,8 @@ draw_tile_flipX_texture (cairo_t *ct, GpBitmap *bitmap, GpTexture *brush)
 {
 	cairo_surface_t *original;
 	cairo_surface_t *texture;
-	cairo_pattern_t *pat;
-	cairo_matrix_t *tempMatrix;
+	cairo_pattern_t *pat, *pattern;
+	cairo_matrix_t tempMatrix;
 	GpRect *rect = brush->rectangle;
 	g_return_val_if_fail (rect != NULL, InvalidParameter);
 
@@ -125,15 +126,11 @@ draw_tile_flipX_texture (cairo_t *ct, GpBitmap *bitmap, GpTexture *brush)
 	pat = cairo_pattern_create_for_surface (original);
 	g_return_val_if_fail (pat != NULL, OutOfMemory);
 
-	tempMatrix = cairo_matrix_create ();
-	g_return_val_if_fail (tempMatrix != NULL, OutOfMemory);
-
 	/* texture surface to be created */
-	texture = cairo_surface_create_similar (original, bitmap->cairo_format,
+	texture = cairo_surface_create_similar (original, from_cairoformat_to_content (bitmap->cairo_format),
 						2 * rect->Width, rect->Height);
 
 	if (texture == NULL) {
-		cairo_matrix_destroy (tempMatrix);
 		return OutOfMemory;
 	}
 
@@ -141,8 +138,8 @@ draw_tile_flipX_texture (cairo_t *ct, GpBitmap *bitmap, GpTexture *brush)
 	{
 		cairo_identity_matrix (ct);
 		/* Draw left part of the texture */
-		cairo_set_target_surface (ct, texture);
-		cairo_set_pattern (ct, pat);
+		ct = cairo_create (texture);
+		cairo_set_source (ct, pat);
 		cairo_rectangle (ct, 0, 0, rect->Width, rect->Height);
 		cairo_fill (ct);
 		
@@ -153,18 +150,19 @@ draw_tile_flipX_texture (cairo_t *ct, GpBitmap *bitmap, GpTexture *brush)
 		/* Not sure if this is a bug, but using rect->Width - 1
 		 * avoids the seam.
 		 */
-		cairo_matrix_translate (tempMatrix, rect->Width - 1, 0);
+		cairo_matrix_translate (&tempMatrix, rect->Width - 1, 0);
 		/* scale in -X direction to flip along X */
-		cairo_matrix_scale (tempMatrix, -1.0, 1.0);
-		cairo_pattern_set_matrix (pat, tempMatrix);
+		cairo_matrix_scale (&tempMatrix, -1.0, 1.0);
+		cairo_pattern_set_matrix (pat, &tempMatrix);
 		cairo_fill (ct);
 	}
 	cairo_restore (ct);
 
-	cairo_surface_set_repeat (texture, 1);
+	pattern = cairo_pattern_create_for_surface (texture);
+	cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
+	
 	brush->pattern = cairo_pattern_create_for_surface (texture);
 
-	cairo_matrix_destroy (tempMatrix);
 	cairo_surface_destroy (texture);
 
 	return gdip_get_status (cairo_status (ct));
@@ -175,8 +173,8 @@ draw_tile_flipY_texture (cairo_t *ct, GpBitmap *bitmap, GpTexture *brush)
 {
 	cairo_surface_t *original;
 	cairo_surface_t *texture;
-	cairo_pattern_t *pat;
-	GpMatrix *tempMatrix;
+	cairo_pattern_t *pat, *pattern;
+	GpMatrix tempMatrix;
 	GpRect *rect = brush->rectangle;
 	g_return_val_if_fail (rect != NULL, InvalidParameter);
 
@@ -188,15 +186,11 @@ draw_tile_flipY_texture (cairo_t *ct, GpBitmap *bitmap, GpTexture *brush)
 	pat = cairo_pattern_create_for_surface (original);
 	g_return_val_if_fail (pat != NULL, OutOfMemory);
 
-	tempMatrix = cairo_matrix_create ();
-	g_return_val_if_fail (tempMatrix != NULL, OutOfMemory);
-
 	/* texture surface to be created */
-	texture = cairo_surface_create_similar (original, bitmap->cairo_format,
+	texture = cairo_surface_create_similar (original, from_cairoformat_to_content (bitmap->cairo_format),
 						rect->Width, 2 * rect->Height);
 
 	if (texture == NULL) {
-		cairo_matrix_destroy (tempMatrix);
 		return OutOfMemory;
 	}
 
@@ -204,8 +198,8 @@ draw_tile_flipY_texture (cairo_t *ct, GpBitmap *bitmap, GpTexture *brush)
 	{
 		cairo_identity_matrix (ct);
 		/* Draw upper part of the texture */
-		cairo_set_target_surface (ct, texture);
-		cairo_set_pattern (ct, pat);
+		ct = cairo_create (texture);
+		cairo_set_source (ct, pat);
 		cairo_rectangle (ct, 0, 0, rect->Width, rect->Height);
 		cairo_fill (ct);
 		
@@ -216,18 +210,18 @@ draw_tile_flipY_texture (cairo_t *ct, GpBitmap *bitmap, GpTexture *brush)
 		/* Not sure if this is a bug, but using rect->Height - 1
 		 * avoids the seam.
 		 */
-		cairo_matrix_translate (tempMatrix, 0, rect->Height - 1);
+		cairo_matrix_translate (&tempMatrix, 0, rect->Height - 1);
 		/* scale in -Y direction to flip along Y */
-		cairo_matrix_scale (tempMatrix, 1.0, -1.0);
-		cairo_pattern_set_matrix (pat, tempMatrix);
+		cairo_matrix_scale (&tempMatrix, 1.0, -1.0);
+		cairo_pattern_set_matrix (pat, &tempMatrix);
 		cairo_fill (ct);
 	}
 	cairo_restore (ct);
 
-	cairo_surface_set_repeat (texture, 1);
+	pattern = cairo_pattern_create_for_surface (texture);
+	cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);	
 	brush->pattern = cairo_pattern_create_for_surface (texture);
 
-	cairo_matrix_destroy (tempMatrix);
 	cairo_surface_destroy (texture);
 
 	return gdip_get_status (cairo_status (ct));
@@ -238,8 +232,8 @@ draw_tile_flipXY_texture (cairo_t *ct, GpBitmap *bitmap, GpTexture *brush)
 {
 	cairo_surface_t *original;
 	cairo_surface_t *texture;
-	cairo_pattern_t *pat;
-	GpMatrix *tempMatrix;
+	cairo_pattern_t *pat, *pattern;
+	GpMatrix tempMatrix;
 	GpRect *rect = brush->rectangle;
 	g_return_val_if_fail (rect != NULL, InvalidParameter);
 
@@ -251,23 +245,19 @@ draw_tile_flipXY_texture (cairo_t *ct, GpBitmap *bitmap, GpTexture *brush)
 	pat = cairo_pattern_create_for_surface (original);
 	g_return_val_if_fail (pat != NULL, OutOfMemory);
 
-	tempMatrix = cairo_matrix_create ();
-	g_return_val_if_fail (tempMatrix != NULL, OutOfMemory);
-
 	/* texture surface to be created */
-	texture = cairo_surface_create_similar (original, bitmap->cairo_format,
+	texture = cairo_surface_create_similar (original, from_cairoformat_to_content (bitmap->cairo_format),
 						2 * rect->Width, 2 * rect->Height);
 
 	if (texture == NULL) {
-		cairo_matrix_destroy (tempMatrix);
 		return OutOfMemory;
 	}
 
 	cairo_save (ct);
 	{
 		cairo_identity_matrix (ct);
-		cairo_set_target_surface (ct, texture);
-		cairo_set_pattern (ct, pat);
+		ct = cairo_create (texture);
+		cairo_set_source (ct, pat);
 
 		/* Draw upper left part of the texture */
 		cairo_rectangle (ct, 0, 0, rect->Width, rect->Height);
@@ -279,24 +269,24 @@ draw_tile_flipXY_texture (cairo_t *ct, GpBitmap *bitmap, GpTexture *brush)
 		/* Not sure if this is a bug, but using rect->Height - 1
 		 * avoids the seam.
 		 */
-		cairo_matrix_translate (tempMatrix, 0, rect->Height - 1);
+		cairo_matrix_translate (&tempMatrix, 0, rect->Height - 1);
 		/* scale in -Y direction to flip along Y */
-		cairo_matrix_scale (tempMatrix, 1.0, -1.0);
-		cairo_pattern_set_matrix (pat, tempMatrix);
+		cairo_matrix_scale (&tempMatrix, 1.0, -1.0);
+		cairo_pattern_set_matrix (pat, &tempMatrix);
 		cairo_fill (ct);
 
 		/* Draw upper right part of the texture */
 		cairo_translate (ct, rect->Width, -rect->Height);
 		cairo_rectangle (ct, 0, 0, rect->Width, rect->Height);
 		/* Reset the pattern matrix and do fresh transformation */
-		cairo_matrix_set_identity (tempMatrix);
+		cairo_matrix_init_identity (&tempMatrix);
 		/* Not sure if this is a bug, but using rect->Width - 1
 		 * avoids the seam.
 		 */
-		cairo_matrix_translate (tempMatrix, rect->Width - 1, 0);
+		cairo_matrix_translate (&tempMatrix, rect->Width - 1, 0);
 		/* scale in -X direction to flip along X */
-		cairo_matrix_scale (tempMatrix, -1.0, 1.0);
-		cairo_pattern_set_matrix (pat, tempMatrix);
+		cairo_matrix_scale (&tempMatrix, -1.0, 1.0);
+		cairo_pattern_set_matrix (pat, &tempMatrix);
 		cairo_fill (ct);
 
 		/* Draw lower right part of the texture */
@@ -305,18 +295,19 @@ draw_tile_flipXY_texture (cairo_t *ct, GpBitmap *bitmap, GpTexture *brush)
 		/* Not sure if this is a bug, but using rect->Height - 1
 		 * avoids the seam.
 		 */
-		cairo_matrix_translate (tempMatrix, 0, rect->Height - 1);
+		cairo_matrix_translate (&tempMatrix, 0, rect->Height - 1);
 		/* scale in -Y direction to flip along Y */
-		cairo_matrix_scale (tempMatrix, 1.0, -1.0);
-		cairo_pattern_set_matrix (pat, tempMatrix);
+		cairo_matrix_scale (&tempMatrix, 1.0, -1.0);
+		cairo_pattern_set_matrix (pat, &tempMatrix);
 		cairo_fill (ct);
 	}
 	cairo_restore (ct);
 
-	cairo_surface_set_repeat (texture, 1);
+	pattern = cairo_pattern_create_for_surface (texture);
+	cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
+	
 	brush->pattern = cairo_pattern_create_for_surface (texture);
 
-	cairo_matrix_destroy (tempMatrix);
 	cairo_surface_destroy (texture);
 
 	return gdip_get_status (cairo_status (ct));
@@ -341,7 +332,7 @@ draw_clamp_texture (cairo_t *ct, GpBitmap *bitmap, GpTexture *brush)
 	cairo_pattern_set_extend (pat, CAIRO_EXTEND_REPEAT);
 
 	/* texture surface to be created */
-	texture = cairo_surface_create_similar (original, bitmap->cairo_format,
+	texture = cairo_surface_create_similar (original, from_cairoformat_to_content (bitmap->cairo_format),
 						rect->Width, rect->Height);
 
 	g_return_val_if_fail (texture != NULL, OutOfMemory);
@@ -349,9 +340,9 @@ draw_clamp_texture (cairo_t *ct, GpBitmap *bitmap, GpTexture *brush)
 	cairo_save (ct);
 	{
 		/* Draw the texture */
-		cairo_set_target_surface (ct, texture);
+		ct = cairo_create (texture);
 		cairo_identity_matrix (ct);
-		cairo_set_pattern (ct, pat);
+		cairo_set_source (ct, pat);
 		cairo_rectangle (ct, 0, 0, rect->Width, rect->Height);
 		cairo_fill (ct);
 	}
@@ -370,18 +361,16 @@ gdip_texture_setup (GpGraphics *graphics, GpBrush *brush)
 {
 	cairo_t *ct;
 	cairo_pattern_t *pattern;
-	cairo_surface_t *temp;
 	GpTexture *texture;
 	GpImage *img, *gr_img;
 	GpBitmap *bmp, *gr_bmp;
 	cairo_format_t format;
-	GpMatrix *product;
+	GpMatrix *product = NULL;
 	unsigned int width;
 	unsigned int height;
 	GpStatus status = Ok;
 
 	g_return_val_if_fail (graphics != NULL, InvalidParameter);
-	g_return_val_if_fail (graphics->copy_of_ctm != NULL, InvalidParameter);
 	g_return_val_if_fail (brush != NULL, InvalidParameter);
 
 	texture = (GpTexture *) brush;
@@ -406,9 +395,6 @@ gdip_texture_setup (GpGraphics *graphics, GpBrush *brush)
 	ct = graphics->ct;
 	g_return_val_if_fail (ct != NULL, InvalidParameter);
 
-	product = cairo_matrix_create ();
-	g_return_val_if_fail (product != NULL, OutOfMemory);
-
 	/* We create the new pattern for brush, if the brush is changed
 	 * or if pattern has not been created yet.
 	 */
@@ -445,6 +431,7 @@ gdip_texture_setup (GpGraphics *graphics, GpBrush *brush)
 		}
 	}
 
+	GdipCreateMatrix (&product);
 	if (status == Ok) {
 		if (texture->pattern == NULL)
 			status = GenericError;
@@ -463,7 +450,7 @@ gdip_texture_setup (GpGraphics *graphics, GpBrush *brush)
 	/*
 		cairo_save (ct);
 		{
-			temp = cairo_surface_create_similar (cairo_current_target_surface (ct),
+			temp = cairo_surface_create_similar (cairo_get_target (ct),
 							     format, width, height);
 			
 			if (temp == NULL) {
@@ -472,7 +459,7 @@ gdip_texture_setup (GpGraphics *graphics, GpBrush *brush)
 			}
 			
 			cairo_set_target_surface (ct, temp);
-			cairo_set_pattern (ct, texture->pattern);
+			cairo_set_source (ct, texture->pattern);
 			cairo_rectangle (ct, 0, 0, width, height);
 			cairo_fill (ct);
 		}
@@ -487,7 +474,7 @@ gdip_texture_setup (GpGraphics *graphics, GpBrush *brush)
 				cairo_matrix_multiply (product, texture->matrix, graphics->copy_of_ctm);
 				cairo_matrix_invert (product);
 				cairo_pattern_set_matrix (pattern, product);
-				cairo_set_pattern (ct, pattern);
+				cairo_set_source (ct, pattern);
 				/*	cairo_pattern_destroy (pattern); */
 			}
 			status = gdip_get_status (cairo_status (ct));
@@ -495,8 +482,6 @@ gdip_texture_setup (GpGraphics *graphics, GpBrush *brush)
 		}
 	}
 
-	cairo_matrix_destroy (product);
-
 	return status;
 }
 
@@ -516,22 +501,15 @@ gdip_texture_clone (GpBrush *brush, GpBrush **clonedBrush)
 	result->base = texture->base;
 	result->wrapMode = texture->wrapMode;
 	result->image = texture->image;
-	result->matrix = cairo_matrix_create ();
 
 	/* Let the clone create its own pattern. */
 	result->pattern = NULL;
 	result->base.changed = TRUE;
 
-	if (result->matrix == NULL) {
-		GdipFree (result);
-		return OutOfMemory;
-	}
-
-	cairo_matrix_copy (result->matrix, texture->matrix);
-	result->rectangle = (GpRect *) malloc (sizeof (GpRect));
+	gdip_cairo_matrix_copy (&result->matrix, &texture->matrix);
+	result->rectangle = (GpRect *) GdipAlloc (sizeof (GpRect));
 
 	if (result->rectangle == NULL) {
-		cairo_matrix_destroy (result->matrix);
 		GdipFree (result);
 		return OutOfMemory;
 	}
@@ -552,10 +530,6 @@ gdip_texture_destroy (GpBrush *brush)
 
 	texture = (GpTexture *) brush;
 
-	if (texture->matrix != NULL)
-		cairo_matrix_destroy (texture->matrix);
-	texture->matrix = NULL;
-
 	if (texture->rectangle != NULL)
 		GdipFree (texture->rectangle);
 	texture->rectangle = NULL;
@@ -573,7 +547,6 @@ GpStatus
 GdipCreateTexture (GpImage *image, GpWrapMode wrapMode, GpTexture **texture)
 {
 	GpBitmap *bitmap;
-	cairo_t *ct;
 	cairo_surface_t *imageSurface;
 
 	g_return_val_if_fail (image != NULL, InvalidParameter);
@@ -583,7 +556,7 @@ GdipCreateTexture (GpImage *image, GpWrapMode wrapMode, GpTexture **texture)
 	else
 		return NotImplemented;
 
-	imageSurface = cairo_surface_create_for_image ((char *)bitmap->data.Scan0,
+	imageSurface = cairo_image_surface_create_for_data ((unsigned char *)bitmap->data.Scan0,
 					bitmap->cairo_format, bitmap->data.Width,
 					bitmap->data.Height, bitmap->data.Stride);
 
@@ -601,7 +574,7 @@ GdipCreateTexture (GpImage *image, GpWrapMode wrapMode, GpTexture **texture)
 
 	(*texture)->wrapMode = wrapMode;
 	(*texture)->image = image;
-	(*texture)->rectangle = (GpRect *) malloc (sizeof (GpRect));
+	(*texture)->rectangle = (GpRect *) GdipAlloc (sizeof (GpRect));
 
 	if ( (*texture)->rectangle == NULL) {
 		cairo_surface_destroy (imageSurface);
@@ -647,13 +620,13 @@ GdipCreateTexture2I (GpImage *image, GpWrapMode wrapMode, int x, int y, int widt
 		  || (bmpWidth < (x + width)) || (bmpHeight < (y + height)))
 	     return OutOfMemory;
 
-	original = cairo_surface_create_for_image ((char *)bitmap->data.Scan0, 
+	original = cairo_image_surface_create_for_data ((unsigned char *)bitmap->data.Scan0, 
 			bitmap->cairo_format, x + width, y + height, bitmap->data.Stride);
 
 	g_return_val_if_fail (original != NULL, OutOfMemory);
 
 	/* texture surface to be used by brush */
-	new = cairo_surface_create_similar (original, bitmap->cairo_format, width, height);
+	new = cairo_surface_create_similar (original, from_cairoformat_to_content (bitmap->cairo_format), width, height);
 
 	if (new == NULL) {
 		cairo_surface_destroy (original);
@@ -661,10 +634,10 @@ GdipCreateTexture2I (GpImage *image, GpWrapMode wrapMode, int x, int y, int widt
 	}
 
 	/* clip the rectangle from original image surface and create new surface */
-	ct = cairo_create ();
-	cairo_set_target_surface (ct, new);
+	ct = cairo_create (new);
 	cairo_translate (ct, -x, -y);
-	cairo_show_surface (ct, original, x + width, y + height);
+	cairo_set_source_surface (ct, original, x + width, y + height);
+	cairo_paint (ct);
 	cairo_destroy (ct);
 	cairo_surface_destroy (original);
 
@@ -679,7 +652,7 @@ GdipCreateTexture2I (GpImage *image, GpWrapMode wrapMode, int x, int y, int widt
 
 	(*texture)->wrapMode = wrapMode;
 	(*texture)->image = image;
-	(*texture)->rectangle = (GpRect *) malloc (sizeof (GpRect));
+	(*texture)->rectangle = (GpRect *) GdipAlloc (sizeof (GpRect));
 
 	if ( (*texture)->rectangle == NULL) {
 		cairo_surface_destroy (new);
@@ -719,7 +692,7 @@ GdipGetTextureTransform (GpTexture *texture, GpMatrix *matrix)
 	g_return_val_if_fail (texture != NULL, InvalidParameter);
 	g_return_val_if_fail (matrix != NULL, InvalidParameter);
 
-	cairo_matrix_copy(matrix, texture->matrix);
+	gdip_cairo_matrix_copy(matrix, &texture->matrix);
 	return Ok;
 }
 
@@ -729,8 +702,7 @@ GdipSetTextureTransform (GpTexture *texture, GDIPCONST GpMatrix *matrix)
 	g_return_val_if_fail (texture != NULL, InvalidParameter);
 	g_return_val_if_fail (matrix != NULL, InvalidParameter);
 
-	texture->matrix = cairo_matrix_create();
-	cairo_matrix_copy(texture->matrix, matrix);
+	gdip_cairo_matrix_copy(&texture->matrix, matrix);
 	texture->base.changed = TRUE;
 
 	return Ok;
@@ -739,15 +711,11 @@ GdipSetTextureTransform (GpTexture *texture, GDIPCONST GpMatrix *matrix)
 GpStatus
 GdipResetTextureTransform (GpTexture *texture)
 {
-	cairo_status_t status;
-	GpStatus s;
 	g_return_val_if_fail (texture != NULL, InvalidParameter);
 
-	status = cairo_matrix_set_identity (texture->matrix);
-	s = gdip_get_status (status);
-	if (s == Ok)
-		texture->base.changed = TRUE;
-	return s;
+	 cairo_matrix_init_identity (texture->matrix);
+	texture->base.changed = TRUE;
+	return Ok;
 }
 
 GpStatus
